#include <google/protobuf/message.h>
#include <scene-persistence/autogen/render.pb.h>
//#include <scene-persistence/autogen/surface_tracking.pb.h>

#include "BAcel.h"
#include "ErrorAcelValue.h"
#include <iostream>


BAcel::BAcel( Message* m ) : message(m), id(0)    // id = 0 means unset
{

}

sra::oid_t BAcel::Id()
{
  return id;
}

bool BAcel::SetId(oid_t id)
{
	if(this->id != 0) return false;
	this->id = id;
	return true;
}

std::shared_ptr<sra::IAcel> BAcel::Property( string name )
{

  const FieldDescriptor* f=0;
  try {
    f = message->GetDescriptor()->FindFieldByName(name);
  } catch (...)
  {
    throw UnsupportedPropertyError("unsupported property " + name);
  }
  if (!f)
    throw UnsupportedPropertyError("unsupported property " + name);

  return PAcel(new BPropertyAcel(message, f));
}

BAcel::~BAcel( void )
{

}

IAcelValue* BAcel::BPropertyAcel::Value()
{
  return this;
}

BAcel::BPropertyAcel::BPropertyAcel(Message* m, const FieldDescriptor*f ) :
message(m), field(f)
{
}

float BAcel::BPropertyAcel::GetFloat()
{
  try 
  {
    return message->GetReflection()->GetFloat(*message, field);
  }
  catch(...) 
  {
    return ErrorAcelValue::GetFloat();
  }
}

void BAcel::BPropertyAcel::SetFloat(float val)
{
  try
  {
    message->GetReflection()->SetFloat(message, field, val);
  }
  catch(...)
  {
    ErrorAcelValue::SetFloat(val);
  }
  return;
}

int32_t BAcel::BPropertyAcel::GetInt()
{
  try 
  {
    return message->GetReflection()->GetInt32(*message, field);
  }
  catch(...) 
  {
    return ErrorAcelValue::GetInt();
  }
}

void BAcel::BPropertyAcel::SetInt(int val)
{
  try
  {
    message->GetReflection()->SetInt32(message, field, val);
  }
  catch(...)
  {
    ErrorAcelValue::SetInt(val);
  }
  return;
}

string BAcel::BPropertyAcel::GetString()
{
  try 
  {
    return message->GetReflection()->GetString(*message, field);
  }
  catch(...) 
  {
    return ErrorAcelValue::GetString();
  }
}

void BAcel::BPropertyAcel::SetString(string val)
{
  try
  {
    message->GetReflection()->SetString(message, field, val);
  }
  catch(...)
  {
    ErrorAcelValue::SetString(val);
  }
  return;
}


bool BAcel::BPropertyAcel::GetBool()
{
  try 
  {
    return message->GetReflection()->GetBool(*message, field);
  }
  catch(...) 
  {
    return ErrorAcelValue::GetBool();
  }
}

void BAcel::BPropertyAcel::SetBool(bool val)
{
  try
  {
    message->GetReflection()->SetBool(message, field, val);
  }
  catch(...)
  {
    ErrorAcelValue::SetBool(val);
  }
  return;
}

sra::float3_t BAcel::BPropertyAcel::GetFloat3()
{
  try 
  {
    const Message& m = message->GetReflection()->GetMessage(*message, field);
    ren::Vector v;
    v.CopyFrom(m);
    return float3_t(v.x(), v.y(), v.z());
  }
  catch(...)
  {
    return ErrorAcelValue::GetFloat3();
  }
}

void BAcel::BPropertyAcel::SetFloat3(sra::float3_t val)
{
  try
  {
	ren::Vector* v = (ren::Vector*) message->GetReflection()->MutableMessage(message, field);
	v->set_x(val.x);
	v->set_y(val.y);
	v->set_z(val.z);
  }
  catch(...)
  {
    ErrorAcelValue::SetFloat3(val);
  }
  return;
}

sra::byte_buffer_t BAcel::BPropertyAcel::GetByteBuffer()
{
  try {
    string scratch;
    auto& f  = message->GetReflection()->GetStringReference(*message, field, &scratch);
    assert(scratch.empty());
    return byte_buffer_t(f.size(), (uint8_t*)f.c_str());
  } catch(...)
  {
    return ErrorAcelValue::GetByteBuffer();
  }
}

void BAcel::BPropertyAcel::SetByteBuffer(sra::byte_buffer_t val)
{
  try {
    message->GetReflection()->SetString(message, field, (const char* ) val.Ptr());
  } catch(...)
  {
    ErrorAcelValue::SetByteBuffer(val);
  }
  return;
}

sra::float_buffer_t BAcel::BPropertyAcel::GetFloatBuffer()
{
  try {
    auto& f  = message->GetReflection()->GetRepeatedField<float>(*message, field);
    return float_buffer_t(f.size(), f.data());
  }
  catch(...)
  {
    return ErrorAcelValue::GetFloatBuffer();
  }
}

void BAcel::BPropertyAcel::SetFloatBuffer(sra::float_buffer_t val)
{
  try {
    auto f  = message->GetReflection()->MutableRepeatedField<float>(message, field);
    for(unsigned int i = 0; i < val.Size(); i++) {
      f->Add(val.Ptr()[i]);
    }
  }
  catch(...)
  {
    ErrorAcelValue::SetFloatBuffer(val);
  }
  return;
}

sra::int_buffer_t BAcel::BPropertyAcel::GetIntBuffer()
{
  try {
  auto& f  = message->GetReflection()->GetRepeatedField<int>(*message, field);
  return int_buffer_t(f.size(), f.data());
  }
  catch(...)
  {
    return ErrorAcelValue::GetIntBuffer();
  }
}

void BAcel::BPropertyAcel::SetIntBuffer(sra::int_buffer_t val)
{
  try {
    auto f  = message->GetReflection()->MutableRepeatedField<float>(message, field);
    for(unsigned int i = 0; i < val.Size(); i++) {
      f->Add(val.Ptr()[i]);
    }
  }
  catch(...)
  {
    ErrorAcelValue::SetIntBuffer(val);
  }
  return;
}

