#include <scene-persistence/autogen/scenedb.pb.h>
#include "XAcel.h"
#include "XAcelValue.h"
#include "BAcel.h"

#include <iostream>
#include <vector>

using std::shared_ptr;

using google::protobuf::Message;
using google::protobuf::FieldDescriptor;
using google::protobuf::Reflection;
//using google::protobuf::Descriptor::ExtensionRange;


oid_t XAcel::Id()
{
  return acel.has_id() ? acel.id() : 0;
}

bool XAcel::SetId(oid_t id)
{
	if(acel.has_id()) return false;
	acel.set_id(id);
	return true;
}

IAcelList* XAcel::List()
{
  return &xlist;
}

PAcel XAcel::AddChild(oid_t id)
{
  PAcel ch = PAcel(CreateAcel(*acel.add_children()));
  ch->SetId(id);
  /*if(!ch->SetId(id)) {
    acel.mutable_children()->RemoveLast();
    return ;
  } */
  list.push_back(ch);
  return ch;
}

PAcel XAcel::AddChild(oid_t id, pb::Acel* a) {
  acel.mutable_children()->AddAllocated(a);

  // set id directly in pb::Acel object
  a->set_id(id);
  PAcel ch = PAcel(CreateAcel(*a));

  list.push_back(ch);
  return ch;
}

#ifdef experiment

PAcel XAcel::AddChild(oid_t id, const Descriptor* desc)
{

  pb::Acel* a = acel.add_children();

  const Descriptor * acelDesc = a->GetDescriptor();

  std::cout << "Num. fields: " << acelDesc->field_count() << std::endl;

  // cycles only through regular fields, not extensions
  for(int i = 0; i < acelDesc->field_count(); i++) {      // Descriptor::extension_range(;   const FieldDescriptor * extension()
	const FieldDescriptor* f = acelDesc->field(i);
	const Descriptor* fieldDesc = f->message_type();   //  const Descriptor* extension_scope() const;

	if(fieldDesc == NULL) {
	  std::cout << "not a message" << std::endl;
	  continue;
    }

	std::cout << "message: " << fieldDesc->full_name() << std::endl;


	// find the message field which has the same type as parameter desc
	if(fieldDesc->full_name() == desc->full_name()) {
       std::cout << "FOUND" << std::endl;

       // sets the specific field
       a->GetReflection()->MutableMessage(a, f);
	}
  }

  std::cout << "Num. extensions: " << acelDesc->extension_count() << std::endl;
  const Descriptor::ExtensionRange * rng = acelDesc->extension_range(0);

  // a->MutableExtension(st::st_camera);  explicit definition works

  /*const FieldDescriptor * f = acelDesc->FindExtensionByName("surface_tracking_pb2.st_camera");
  if(f == NULL) std::cout << "undefined ";
  else std::cout << f->full_name();*/

  // cycles only through extensions
  //for(int i = 0; i < acelDesc->extension_count(); i++) {
 /* for(int j = rng->start; j < rng->end; j++) {
  	std::cout << j << " ";
  	const FieldDescriptor* f = NULL;
  	f = acelDesc->field(j);
  	if(f == NULL) std::cout << "undefined ";
  	else std::cout << f->full_name();  */
    //const Descriptor* fieldDesc = f->message_type();

  	/*if(fieldDesc == NULL) {
  	  std::cout << "not a message" << std::endl;
  	  continue;
      }

  	std::cout << "message: " << fieldDesc->full_name() << std::endl;


  	// find the message field which has the same type as parameter desc
  	if(fieldDesc->full_name() == desc->full_name()) {
         std::cout << "FOUND" << std::endl;

         // sets the specific field
         a->GetReflection()->MutableMessage(a, f);
  	}*/
 // }


  // set id directly in pb::Acel object
  a->set_id(id);
  PAcel ch = PAcel(CreateAcel(*a));

  list.push_back(ch);
  return ch;
}

#endif

bool XAcel::DeleteChild(size_t index)
{
  if((int)index < acel.mutable_children()->size())
  {
	acel.mutable_children()->DeleteSubrange(index, 1);

	if(index < list.size())
	{
		list.erase(list.begin() + index);
		return true;
	}
	else
	{
		std::cout << "XAcel list out of sync with pb::Acel list (XAcel::DeleteChildIndex)" << std::endl;
	}

  }

  return false;
}

bool XAcel::DeleteChildId(oid_t id)
{
  // find an index of child if it exists
  size_t i = 0;
  for(; i < list.size(); ++i) {
    if( list[i]->Id() == id ) {

      // delete child
      list.erase(list.begin() + i);
      acel.mutable_children()->DeleteSubrange(i, 1);

      return true;
    }
  }

  return false;
}

PAcel XAcel::Property( string name )
{
  //TODO: naive implementation should be replaced with a map or smth
  for (
    auto p = acel.mutable_properties()->begin();
    p!= acel.mutable_properties()->end(); p++ )
    if (p->name() == name)
    {
      return PAcel(CreateAcel(*p->mutable_value()));
    }
    throw UnsupportedPropertyError("xacel unsupported property " + name);
}

PAcel XAcel::AddProperty(string name, oid_t id)
{
  // check existence of property
  for(auto p = acel.mutable_properties()->begin();
	   p != acel.mutable_properties()->end(); p++ )
  {
	if (p->name() == name)
	  throw ExistingPropertyError("xacel existing property " + name);
  }

  pb::Property* p = acel.add_properties();
  // set name
  p->set_name(name);
  // wrap property acel and set id
  PAcel ch = PAcel(CreateAcel(*p->mutable_value()));
  ch->SetId(id);
  return ch;
}

PAcel XAcel::AddProperty(string name, oid_t id, pb::Acel* a) {

  // check existence of property
  for(auto p = acel.mutable_properties()->begin();
	   p != acel.mutable_properties()->end(); p++ )
  {
	if (p->name() == name)
	  throw ExistingPropertyError("xacel existing property " + name);
  }

  pb::Property* p = acel.add_properties();
  // set name
  p->set_name(name);
  // set acel value
  p->set_allocated_value(a);

  // set id directly in pb::Acel object
  a->set_id(id);

  PAcel ch = PAcel(CreateAcel(*a));
  return ch;
}


bool XAcel::DeleteProperty(string name)
{
  // check existence of property
  size_t index = 0;
  for(auto p = acel.mutable_properties()->begin();
	   p != acel.mutable_properties()->end(); index++, p++ )
  {
	if (p->name() == name)
	{
	  acel.mutable_properties()->DeleteSubrange(index, 1);
	  return true;
	}
  }

  return false;
}

IAcelValue* XAcel::Value()
{
  if (acel.has_value())
  {
    if (!PtrValue)
      PtrValue = shared_ptr<IAcelValue>(new XAcelValue(*acel.mutable_value()));
    return PtrValue.get();
  }
  else
    return &ErrorAcelValue::Instance;
}

IAcelValue* XAcel::SetValue()
{
  acel.clear_value();
  PtrValue = shared_ptr<IAcelValue>(new XAcelValue(*acel.mutable_value()));;
  return PtrValue.get();
}

void XAcel::ClearValue()
{
  PtrValue.reset();
  acel.clear_value();
}

XAcel::XAcel(pb::Acel& a) : acel(a), xlist(NULL)
{
    for (
      auto p = acel.mutable_children()->begin();
      p != acel.mutable_children()->end(); p++ )
    {
      list.push_back(PAcel(CreateAcel(*p)));
    }

    xlist = XAcelList(&list);

}

sra::IAcel* XAcel::CreateAcel(pb::Acel& a)
{

  // keep special handling of renderer-specific messages for a time being
  if (a.has_ren_camera())
  {
	BAcel* na = new BAcel(a.mutable_ren_camera());
	na->SetId(a.id());
	return na;
  }
  if (a.has_ren_depth())
  {
  	BAcel* na = new BAcel(a.mutable_ren_depth());
  	na->SetId(a.id());
  	return na;
  }
  if (a.has_ren_geometry())
  {
  	BAcel* na = new BAcel(a.mutable_ren_geometry());
  	na->SetId(a.id());
  	return na;
  }
  if (a.has_ren_rgb())
  {
   	BAcel* na = new BAcel(a.mutable_ren_rgb());
   	na->SetId(a.id());
   	return na;
  }
  if (a.has_ren_spotlight())
  {
	BAcel* na = new BAcel(a.mutable_ren_spotlight());
	na->SetId(a.id());
	return na;
  }

  // List all fields of the message which are currently set (includes extensions)
  const Reflection *reflection = a.GetReflection();
  std::vector< const FieldDescriptor * > setFields;
  reflection->ListFields(a, &setFields);

  // test how many extensions and original data fields are set
  int index = 0, i = 0;
  int fieldCount = 0, extCount = 0;
  for(std::vector< const FieldDescriptor * >::iterator it = setFields.begin(); it != setFields.end(); ++it, ++i)
  {
	if((*it)->is_extension())
	{
	  index = i;
	  ++extCount;
	}
	else ++fieldCount;
  }

  if((extCount == 1) && (fieldCount == 1))  // fieldCount == 1  because of id field
  {
	// retrieve a single custom message and create BAcel
	Message *msg = reflection->MutableMessage(&a, setFields[index]);
	BAcel* na = new BAcel(msg);
	na->SetId(a.id()); // implant ID into BAcel from pb::Acel because only specified field message is passed into BAcel
    return na;
  }
  else if((extCount == 0) && (fieldCount >= 0))  // before (fieldCount > 0) doesn't allow an empty acel
  {
	// no custom protobuf message available, so structure through properties and children assumed
	return new XAcel(a);
  }
  else
  {
	throw InvalidAcelError("XAcel/BAcel: Acel contains other data apart from a single custom protobuf message.");
  }
}


//sra::PAcel MessageAcel::Property( string name )
//{
//  this->message->GetDescriptor()
//  const FieldDescriptor* field = dst->GetDescriptor()->FindFieldByName(name);
//  return PAcel(new MessageFieldAcel(shared_ptr<Message>(this->message), shared_ptr<FieldDescriptor>(field)));
//}
