#pragma  once
#include "sra-api/sra_scene.h"

#include "XAcelList.h"
#include "ErrorAcelValue.h"
#include <google/protobuf/message.h>
#include <google/protobuf/descriptor.h>

using google::protobuf::Descriptor;

namespace pb
{
  class Acel;
}
using namespace sra;

class XAcel: public IAcel
{
  pb::Acel& acel;  // all data are inside this object

  // following members are only data placeholders compatible with SRA API
  std::vector<PAcel> list;  // to make more efficient List() method, wrapping of child acels into XAcels is stored here
  XAcelList xlist;   // encapsulates list to give in IAcelList interface. List() returns a pointer to this.
  std::shared_ptr<IAcelValue> PtrValue;  

public:

  XAcel(pb::Acel& a);

  /**
   * Factory method which creates XAcel or BAcel object depending on the contents of Acel protobuf message.
   * If the acel data are structured through the data fields properties and children, XAcel is created.
   * If the acel data are structured according to an optional data field containing custom protobuf message, BAcel is created.
   */
  static IAcel* CreateAcel(pb::Acel& a);

  /**
   * Return 0 if the id is not defined.
   */
  virtual oid_t Id();
  virtual bool SetId(oid_t id);
  virtual IAcelList* List();
  virtual PAcel AddChild(oid_t id);
//  virtual PAcel AddChild(oid_t id, const Descriptor* d);
  virtual PAcel AddChild(oid_t id, pb::Acel* a);
  virtual bool DeleteChild(size_t index);
  virtual bool DeleteChildId(oid_t id);

  virtual PAcel Property( string name );
  virtual PAcel AddProperty(string name, oid_t id);
  virtual PAcel AddProperty(string name, oid_t id, pb::Acel* a);
  virtual bool DeleteProperty(string name);

  virtual IAcelValue* Value();
  /**
   * Enable value, so it can be used. If the value has been set, the content is reset.
   */
  virtual IAcelValue* SetValue();
  /**
   * Clears and invalidates value. SetValue() needs to be called to define a value of some type again.
   */
  void ClearValue();

  virtual ~XAcel() { }
  
};
 
