#include "XStream.h"
#include "XAcel.h"

#include <scene-persistence/autogen/scenedb.pb.h>
#include <scene-persistence/autogen/stream_fsio.pb.h>
#include <scene-persistence/depth_loader.h>
#include <scene-streaminput/texture_loader_api.h>
#include "BAcel.h"


template <typename T>
std::shared_ptr<T> make_auto_ptr(T* o) {return std::shared_ptr<T>(o); }

// what is it good for?
#pragma region auxilary acel classes

class ErrorAcelErrorValue;
class DictionaryAcel : public ErrorAcel
{
  std::map<string, std::shared_ptr<ErrorAcelErrorValue> > dict;
public:
  void Add(string name, std::shared_ptr<ErrorAcelErrorValue> val);
  virtual PAcel Property( string name);
};

class ErrorAcelErrorValue : public ErrorAcelValue,  public ErrorAcel {};
class BasicBytesAcelValue : public ErrorAcelErrorValue
{
  byte_buffer_t bytes;
public:
  BasicBytesAcelValue(byte_buffer_t b): bytes(b) {}
  virtual byte_buffer_t GetByteBuffer()  { return bytes; }
  virtual IAcelValue* Value() { return this; }
};
class BasicIntValue : public ErrorAcelErrorValue
{
  int value;
public:
  BasicIntValue(int b): value(b) {}
  virtual int GetInt()  { return value; }
  virtual IAcelValue* Value() { return this; }
};

class BasicFloatsAcelValue : public ErrorAcelErrorValue
{
  float_buffer_t floats;
public:
  BasicFloatsAcelValue(float_buffer_t f) : floats(f) { }
  virtual float_buffer_t GetFloatBuffer() { return floats; }
  virtual IAcelValue* Value() { return this; }
};



void DictionaryAcel::Add( string name, std::shared_ptr<ErrorAcelErrorValue> val )
{
  dict[name] = val;
}

PAcel  DictionaryAcel::Property( string name )
{
  auto p = dict.find(name);
  if (dict.end() == p)
  {
    std::cout << "unsupported property " << name;
    throw UnsupportedPropertyError();
  }
  return p->second;
}


#pragma endregion


PAcel XStream::GetStreamGlobals()
{
  return PAcel(XAcel::CreateAcel(*stream.mutable_stream_globals()));
}

#if 0
void XStream::Reset()
{
  finished = false;
  frame_index = -1;

  int first_frame = (stream.HasExtension(fsio::first_frame)) ?
                     stream.GetExtension(fsio::first_frame) : -1;
  int  last_frame = (stream.HasExtension(fsio:: last_frame)) ?
                     stream.GetExtension(fsio:: last_frame) : -1;
  int  frame_step = (stream.HasExtension(fsio:: frame_step)) ?
                     stream.GetExtension(fsio:: frame_step) :  1;

  if (stream.HasExtension(fsio::exr_channel))
  {
    // extract a few channels from a multichannel EXR image

    int n_channels = stream.ExtensionSize(fsio::exr_channel);
    std::vector< std::string > channel_names(n_channels);

    for (int ch = 0; ch < n_channels; ++ch)
      channel_names[ch] = stream.GetExtension( fsio::exr_channel, ch);

    // call TextureLoader constructor passing channel_names 
    TexLoader = make_auto_ptr(
      ITextureLoader::CreateTextureLoader(work_dir + stream.video(),
      first_frame, last_frame, frame_step, channel_names));
  }
  else
    TexLoader = make_auto_ptr(
      ITextureLoader::CreateTextureLoader(work_dir + stream.video(),
      first_frame, last_frame, frame_step));
}
#else
void XStream::Reset()
{
  finished = false;
  frame_index = -1;
  TexLoader = ITextureLoader::CreateTextureLoader(stream, work_dir);
}
#endif

void XStream::Initialize(std::shared_ptr<sra::IAcel> data)
{
  /* We restrict the use of the Initialize member function only to 
   * initialize uninitialized stream objects.  This is only to simplify 
   * the semantics of the function. This function could be used at any
   * timea during the life of the stream to change it's data (file names,
   * names of exr channels, first frame, etc). 
   *
   * After a stream has been intialized, its data cannot be changed.
   * Only modifying frames or adding frames at the end of the stream. */
  if (TexLoader)
	  throw std::runtime_error("attempting to initialize an already initialized XStream");

  // convert acel properties to pb::Stream fields
  
  // mandatory field
  stream.set_video(data->Property("video")->Value()->GetString());

  // optional fields - silent UnsupportedPropertyError
  try { stream.set_depth(data->Property("depth")->Value()->GetString()); }
  catch (UnsupportedPropertyError& e) {}

  try { stream.SetExtension(fsio::name, data->Property("name")->Value()->GetString()); }
  catch (UnsupportedPropertyError &e) {}

  try { stream.SetExtension(fsio::first_frame, data->Property("first_frame")->Value()->GetInt()); }
  catch (UnsupportedPropertyError &e) {}

  try { stream.SetExtension(fsio::last_frame, data->Property("last_frame" )->Value()->GetInt()); }
  catch (UnsupportedPropertyError &e) {}

  try { stream.SetExtension(fsio::frame_step, data->Property("frame_step" )->Value()->GetInt()); }
  catch (UnsupportedPropertyError &e) {}

//TODO camera field

  // optional field exr_channel is a vector of strings
  try
  {
    PAcel exr_channels = data->Property("exr_channel");

    for (size_t ch = 0; ch < exr_channels->List()->Count(); ++ch)
		stream.AddExtension(fsio::exr_channel,
				exr_channels->List()->Get(ch)->Value()->GetString());
  }
  catch (UnsupportedPropertyError &e) {}

  Reset();
}


bool XStream::AdvanceToNextFrame()
{
  if (finished)
	  throw std::runtime_error("Calling XStream::AdvanceToNextFrame"
                              " of an already finished Stream");
  if (!TexLoader)
	  throw std::runtime_error("Calling XStream::AdvanceToNextFrame"
                              " with a null TexLoader");

  finished = !(TexLoader->NextFrame());
  ++frame_index;
  return (!finished);
}

#ifdef USE_XACEL_IN_XSTREAM
/* A failed attemp to use XAcel to output the frame (as opposed to outputting a
 * DictionaryAcel) 
 *
 * The problem with the XAcel is that it has pb::Acel& as member. In the code bellow
 * I create pb::Acel and a XAcel which has as member a reference to it.
 *
 * The problem is that XAcel only contains a member to the pb::Acel. When we
 * quit the function the pb::Acel is destroyed, which leaves the reference pointing
 * to a invalid memory location.
 */
PAcel XStream::GetCurrentFrame()
{
  // create a XAcel
  pb::Acel a;
  auto frame_acel = new XAcel(a);
  sra::oid_t id = 2;

  frame_acel->AddProperty(XSTREAM_FRAME_VIDEO_DATA,    id++);
  frame_acel->AddProperty(XSTREAM_INFO_VIDEO_WIDTH,    id++);
  frame_acel->AddProperty(XSTREAM_INFO_VIDEO_HEIGHT,   id++);
  frame_acel->AddProperty(XSTREAM_INFO_VIDEO_CHANNELS, id++);

  if (!finished)
  {
    byte_buffer_t frame_data(TexLoader->Size(), TexLoader->GetCurrentData());
    frame_acel->Property(XSTREAM_FRAME_VIDEO_DATA   )->SetValue()->SetByteBuffer(frame_data);
    frame_acel->Property(XSTREAM_INFO_VIDEO_WIDTH   )->SetValue()->SetInt(TexLoader->width   );
    frame_acel->Property(XSTREAM_INFO_VIDEO_HEIGHT  )->SetValue()->SetInt(TexLoader->height  );
    frame_acel->Property(XSTREAM_INFO_VIDEO_CHANNELS)->SetValue()->SetInt(TexLoader->channels);

    if (DepthLoader)
    {
      float_buffer_t depth(
        DepthLoader->width()* DepthLoader->height(), 
        DepthLoader->depth_stream(frame_index % DepthLoader->depth_stream_size()).values().data() );   

		frame_acel->AddProperty(XSTREAM_FRAME_DEPTH_DATA,  id++);
		frame_acel->AddProperty(XSTREAM_INFO_DEPTH_WIDTH,  id++);
		frame_acel->AddProperty(XSTREAM_INFO_DEPTH_HEIGHT, id++);

      frame_acel->Property(XSTREAM_FRAME_DEPTH_DATA )->SetValue()->SetFloatBuffer(depth);
      frame_acel->Property(XSTREAM_INFO_DEPTH_WIDTH )->SetValue()->SetInt(DepthLoader->width ());
      frame_acel->Property(XSTREAM_INFO_DEPTH_HEIGHT)->SetValue()->SetInt(DepthLoader->height());
    }
  }
  else
  {
    // return empty frame acel
    frame_acel->Property(XSTREAM_FRAME_VIDEO_DATA   )->SetValue()->SetByteBuffer(byte_buffer_t(0, 0x0));
    frame_acel->Property(XSTREAM_INFO_VIDEO_WIDTH   )->SetValue()->SetInt(-1);
    frame_acel->Property(XSTREAM_INFO_VIDEO_HEIGHT  )->SetValue()->SetInt(-1);
    frame_acel->Property(XSTREAM_INFO_VIDEO_CHANNELS)->SetValue()->SetInt(-1);
  }

  return PAcel(frame_acel);
}
#else
PAcel XStream::GetCurrentFrame()
{
	// create an XAcel
  auto acel = new DictionaryAcel();
  if (!finished)
  {
    if (DepthLoader)
    {
      float_buffer_t depth(
        DepthLoader->width()* DepthLoader->height(), 
        DepthLoader->depth_stream(frame_index % DepthLoader->depth_stream_size()).values().data() );   
      acel->Add(XSTREAM_FRAME_DEPTH_DATA, make_auto_ptr(new BasicFloatsAcelValue(depth)));

      acel->Add(XSTREAM_INFO_DEPTH_WIDTH,  make_auto_ptr(new BasicIntValue(DepthLoader->width())));
      acel->Add(XSTREAM_INFO_DEPTH_HEIGHT, make_auto_ptr(new BasicIntValue(DepthLoader->height())));
    }

    acel->Add(XSTREAM_INFO_VIDEO_WIDTH,    make_auto_ptr(new BasicIntValue(TexLoader->Width()     )));
    acel->Add(XSTREAM_INFO_VIDEO_HEIGHT,   make_auto_ptr(new BasicIntValue(TexLoader->Height()    )));
    acel->Add(XSTREAM_INFO_VIDEO_CHANNELS, make_auto_ptr(new BasicIntValue(TexLoader->Channels()  )));
    acel->Add(XSTREAM_INFO_VIDEO_DEPTH,    make_auto_ptr(new BasicIntValue(TexLoader->PixelDepth())));
    acel->Add(XSTREAM_INFO_VIDEO_INDEX,    make_auto_ptr(new BasicIntValue(TexLoader->FrameIndex())));

    byte_buffer_t rgb(TexLoader->Size(), TexLoader->GetCurrentData());
    acel->Add(XSTREAM_FRAME_VIDEO_DATA, make_auto_ptr(new BasicBytesAcelValue(rgb)));
  }
  else
  {
    // return empty frame acel
    acel->Add(XSTREAM_INFO_VIDEO_WIDTH,    make_auto_ptr(new BasicIntValue(-1)));
    acel->Add(XSTREAM_INFO_VIDEO_HEIGHT,   make_auto_ptr(new BasicIntValue(-1)));
    acel->Add(XSTREAM_INFO_VIDEO_CHANNELS, make_auto_ptr(new BasicIntValue(-1)));
    acel->Add(XSTREAM_INFO_VIDEO_DEPTH,    make_auto_ptr(new BasicIntValue(-1)));
    acel->Add(XSTREAM_INFO_VIDEO_INDEX,    make_auto_ptr(new BasicIntValue(-1)));
    acel->Add(XSTREAM_FRAME_VIDEO_DATA,    make_auto_ptr(new BasicBytesAcelValue
                                                       (byte_buffer_t(0, 0x0))));
  }
  return PAcel(acel);
}
#endif

void XStream::SetCurrentFrame(PAcel frame)
{
  if (DepthLoader) { /* TODO: writting for 'depth-in-protobuf' format */ }

  // check for these props in acel
  int w = frame->Property(XSTREAM_INFO_VIDEO_WIDTH   )->Value()->GetInt();
  int h = frame->Property(XSTREAM_INFO_VIDEO_HEIGHT  )->Value()->GetInt();
  int c = frame->Property(XSTREAM_INFO_VIDEO_CHANNELS)->Value()->GetInt();
  int d;
  try { d = frame->Property(XSTREAM_INFO_VIDEO_DEPTH   )->Value()->GetInt(); }
  catch (UnsupportedPropertyError& e) { d = -1; }


  const uint8_t* data = frame->Property(XSTREAM_FRAME_VIDEO_DATA)
                             ->Value()->GetByteBuffer().Ptr();

  TexLoader->SetCurrentData(data, w, h, c, d);

  // set video as unfinished and update last frame
  if (finished) finished = false;
}


XStream::XStream(pb::Stream& s, std::string work_dir) : stream(s), work_dir(work_dir)
{
  Reset();
  if (stream.has_depth())
    DepthLoader = GetDepthStream(work_dir + stream.depth());
}

std::shared_ptr<sra::IAcel> XStream::GetCamera()
{
  return make_auto_ptr(new BAcel(stream.mutable_camera()));   //TODO: setting id?
}

std::string XStream::GetName()
{
  if(!stream.HasExtension(fsio::name))
	  UnsupportedPropertyError("Name of the stream is not defined.");
  return  stream.GetExtension(fsio::name);
}

std::string XStream::GetStreamPath()
{
  if(!stream.has_video())
	  UnsupportedPropertyError("The stream path is not defined.");
  return  stream.video();
}

int XStream::GetFirstFrameIndex()
{
  return (stream.HasExtension(fsio::first_frame)) ?
          stream.GetExtension(fsio::first_frame)  : -1;
}

int XStream::GetLastFrameIndex()
{
  return (stream.HasExtension(fsio::last_frame)) ?
          stream.GetExtension(fsio::last_frame)  : -1;
}

int XStream::GetFrameIndexStep()
{
  return (stream.HasExtension(fsio::frame_step)) ?
          stream.GetExtension(fsio::frame_step)  : -1;
}

