#pragma  once
#include "sra-api/sra_scene.h"
#include "sra_implementation.h"

namespace pb
{
  class Stream;
}

using namespace sra;

class ITextureLoader;
class Data;

class XStream: public IStream
{
private:

  pb::Stream & stream;
  int frame_index;
  /* frame_index is only used to load depth stored in a protobuf
	* as an array of floats. It must start at 0. */
  bool finished;
  std::shared_ptr<ITextureLoader> TexLoader;
  std::shared_ptr<Data> DepthLoader;
  std::string work_dir;

public:

  // builds a XStream wrapper around a pb::Stream
  XStream(pb::Stream& s, std::string work_dir);
  ~XStream() {}

  // get acel with stream global metadata
  std::shared_ptr<sra::IAcel> GetStreamGlobals();

  // sets stream data. Use when adding a new stream to a scene
  void Initialize(std::shared_ptr<sra::IAcel> stream_data);

  // sets XStream to initial frame
  void Reset();

  // loads next frame in buffer; returns false if there is no next frame
  virtual bool AdvanceToNextFrame();

  // creates an acel containing frame dimensions and data 
  virtual std::shared_ptr<sra::IAcel> GetCurrentFrame();

  // overwrites current frame with data. If current frame points after the end
  // of stream, it adds a new frame.
  virtual void SetCurrentFrame(std::shared_ptr<sra::IAcel> frame_data);

  // gets stream camera
  virtual std::shared_ptr<sra::IAcel> GetCamera();

  // gets name of the stream
  std::string GetName();

  // gets path of the stream
  std::string GetStreamPath();

  // get frame indexing information (-1 if implementation does not support it)
  virtual int GetFirstFrameIndex();
  virtual int GetLastFrameIndex();
  virtual int GetFrameIndexStep();
};
