#pragma  once

#ifndef SWIG
#include <stdexcept>
#include <string>
#include <memory>
#endif

#include <scene-core/sra-api/sra_scene.h>
namespace sra
{
  using std::string;
  class IConverter;

  // interface to access a computational module
  class IModule 
  {
  public:
    // execute the module. 
    // If not all required inputs are set, the InputNotSetError is raised with the name of one of the inputs.
    // returns the results of the computation in the form of an acel
    virtual std::shared_ptr<IAcel> Execute()  = 0; 
    virtual ~IModule() {}
  };


  // context modules affect the execution of the other modules inside Begin and End blocks
  class IContextModule : public IModule
  {
  public:
    virtual void Begin() = 0;
    virtual void End() = 0;
    virtual ~IContextModule() {}
  };

  // exception raised if one or more inputs are not set
  struct InputNotSetError : public std::exception
  {
    string name;
    InputNotSetError (string name) : name(name) {}
  }; 

  // converts an acel to a specific module representation
  // and loads the data into the input of the module
  class IConverter 
  {
  public:
    /// loads the data from acel into the associated module input
    /// if the acel can not be converted to the data structure required by the module input, 
    /// the exception  UnsupportedTypeError  is thrown 
    virtual void Assign(std::shared_ptr<IAcel> value) = 0;
    virtual ~IConverter() {}
  };
  /// keeps track of module-specific cache objects created from acels
}