#include "sra_implementation.h"
#include <scene-persistence/autogen/scenedb.pb.h>
#include <fstream>
#include <stdexcept>
#include <iostream>

#include "XScene.h"

using std::shared_ptr;

namespace sra
{
  PScene get_scene_representation(string file)
  {
    auto sc = new pb::Scene();
    std::ifstream in(file, std::ios::in|std::ios::binary);
    if (!in.is_open())
    {
      std::cout << "could not find scene file"  << std::endl;
      throw std::invalid_argument(file);
    }

    // extract directory part of scene file path
    string dir = "./";
    string rev(file.rbegin(), file.rend());
    auto pos = rev.find_first_of("/\\");
    if (pos !=  rev.npos)
    {
      assert(pos);
      dir = file.substr(0, file.length() - pos);
    }

    sc->ParseFromIstream(&in);
    return PScene(new XScene(shared_ptr<pb::Scene>(sc), dir));
  }

  /**
   * Creates a new scene object without any contents.
   */
  PScene make_scene_representation(string dir)
  {
    auto sc = new pb::Scene();
    return PScene(new XScene(shared_ptr<pb::Scene>(sc), dir));
  }

  void set_scene_representation(string file, PScene sc, bool generate_text_protobuf)
  {
	string path = sc->GetRootDirectory();

	if((*path.rbegin()) != '/') path += "/";
	path += file;

	std::ofstream out(path, std::ios::out|std::ios::binary);
	if(!out.is_open())
	{
   	  std::cout << "could not create scene file" << std::endl;
	  throw std::invalid_argument(path);
	}

	std::shared_ptr<pb::Scene> scpb = ((XScene*) sc.get())->GetPBScene();
	scpb->SerializeToOstream(&out);

    // create text description of scene file - it is useful
    if (generate_text_protobuf)
    {
      std::fstream txt_output(path + ".txt", std::ios::out);
      txt_output << scpb->DebugString() ;
      txt_output.close();
    }

	return;
  }

}
