 
JOIN_VERTEX_ATTRIBUTES = True 

class ColladaScene:
    def __init__(self, filename, config):
        
        self.vertex_idx = dict()
        self.vertex_atr= dict()
        self.normal_atr = dict()
        self.textur_atr = dict()
        self.diffuse_texture = dict()
        self.objects = list()
        
        import collada     
        dae = collada.Collada(filename, ignore=[collada.DaeUnsupportedError,
                                               collada.DaeBrokenRefError])
        self.dae = dae
        if dae.scene is not None:
            #load camera
            cam = [c for c in dae.scene.objects('camera')]
            assert(len(cam) <= 1)             
            self.camera = cam[0]
            #load light
            light = [c for c in dae.scene.objects('light')]
            self.light = None
            assert(len(light) <= 1 )             
             
            from pprint import pprint
            if len(light) > 0:
                if type(light[0] ) ==  collada.light.BoundSpotLight:
                    self.light = light[0]
                else:                 
                    print "warning: light type is not supported, only spotlight is currently supported"

                pprint(vars(light[0].original))
            
        if dae.scene is not None and config.import_collada_meshes:
            for geom in dae.scene.objects('geometry'):
                id = geom.original.id
                try:
                    self.diffuse_texture[id] = self.get_texture_path(geom)
                except:
                    pass

                primitives = [ p for p in geom.primitives()]
                assert(len(primitives) == 1)

                for prim in primitives:
                    # use primitive-specific ways to get triangles
                    prim_type = type(prim).__name__
                    if prim_type == 'BoundTriangleSet':
                        triangles = prim
                    elif prim_type == 'BoundPolylist':
                        triangles = prim.triangleset()
                    else:
                        print 'Unsupported mesh used:', prim_type
                        triangles = None
    
                    
                    if triangles is not None:
                        triangles.generateNormals()
                        # We will need flat lists for VBO (batch) initialization
                        self.objects.append(id)
                        
                        vertex_atr = triangles.vertex.flatten().tolist()
                        vertex_idx = triangles.vertex_index.flatten().tolist()
                        normal_atr = triangles.normal.flatten().tolist()                        
                        normal_idx = triangles.normal_index.flatten().tolist()			
                        textur_atr = triangles.texcoordset[0].flatten().tolist()
                        textur_idx = triangles.texcoord_indexset[0].flatten().tolist()
                        
                        if JOIN_VERTEX_ATTRIBUTES:
                            #print vertex_atr
                            self.join_attributes(vertex_atr, vertex_idx, normal_atr, normal_idx, textur_atr, textur_idx)
                           # print vertex_atr
                            self.vertex_idx[id] = vertex_idx
                            self.vertex_atr[id] = vertex_atr
                            self.normal_atr[id] = normal_atr
                            self.textur_atr[id] = textur_atr                            
                        else:                  
                            self.vertex_idx[id] = vertex_idx
                            self.vertex_atr[id] = vertex_atr
                            self.normal_idx[id] = normal_idx
                            self.normal_atr[id] = normal_atr
                            self.textur_idx[id] = textur_idx
                            self.textur_atr[id] = textur_atr  
        
    def join_attributes(self, vertex_atr, vertex_idx, normal_atr, normal_idx, textur_atr, textur_idx):
        imap = dict()
        new_atr = list()
        new_vertex_idx = list()
        
        
        elem_count = len(vertex_idx)
        assert(elem_count == len(normal_idx))
        assert(elem_count == len(textur_idx))
        
        # join attributes and store all unique combinations, indexed by new index 
        for (v, n, t) in zip(vertex_idx, normal_idx, textur_idx):             
            if not imap.has_key((v, n, t)):
                new_atr.append((
                                 vertex_atr[v*3:(v+1)*3] , 
                                 normal_atr[n*3:(n+1)*3] , 
                                 textur_atr[t*2:(t+1)*2]  ))
                imap[(v, n, t)]= len(new_atr) - 1                
            new_vertex_idx.append(imap[(v, n, t)])
            
        #now rewrite the attributes and the vertex index
        vertex_idx[:] = new_vertex_idx 

        vertex_atr[:] = [item for sublist in map(lambda x: x[0], new_atr) for item in sublist]
        normal_atr[:] = [item for sublist in map(lambda x: x[1], new_atr) for item in sublist]
        textur_atr[:] = [item for sublist in map(lambda x: x[2], new_atr) for item in sublist]

    def get_texture_path(self, geom):
        materials = [m.target for  m in  geom.materialnodebysymbol.values()]
        m = materials[0]    
        e  = m.effect
        return e.diffuse.sampler.surface.image.path

def get_scene(file):
    import config
    return ColladaScene(file, config)    

 
   
class basic_conf:
    import_collada_meshes = True

 


def  test_collada():
    scene  = ColladaScene("../../sample_data/twocubes/twocubes.dae", basic_conf)
    # #print(help(scene.dae))
    # #print(dir(scene))    
    
    # from pprint import pprint
    # s = scene.dae.scene
    # g = [geom for geom in s.objects('geometry')]
    # #pprint((g[0].materialnodebysymbol ))
    # materials = [m.target for  m in  g[0].materialnodebysymbol.values()]
    # m = materials[0]
    # #print(name)
    # #print(m.id)

    # #print help(m)
    # e  = m.effect
    # print (help(e))
    # #pprint(vars(e))
    # #print(e.diffuse.sampler.surface.image.path)



#test_collada()