import sys
import path_loader  # imports autogen python classes from scene-persistence
import config 
import os.path
import util_image

from convert_collada import convert_camera, convert_geometry, convert_spotlight 
 

class SRAFile:
    """ represents sra structure on top of the protobuf file"""

    def __init__(self):
        from SRAWriter import SRAWriter
        self.writer = SRAWriter()
        self._camera = self.writer.global_value("camera")
         
        self.patches = self.writer.global_list("patches")
        self.texture_data = self.writer.global_list("textures")
        self.textures = dict()
        self._spotlight = None

    def add_geometry(self):    
        geom  = self.patches.children.add() 
        geom.id =  self.writer.newid()  
        return geom.ren_geometry 
    
    def find_or_create_texture(self, file):
        if not self.textures.has_key(file):
            t = self.texture_data.children.add()
            t.id = self.writer.newid()
            id = len(self.textures) + 1
            path = os.path.join(config.data_folder, file)
            convert_texture(t.ren_rgb, path, id)
            self.textures[file] = id
            
        return self.textures[file]


    def camera(self):
        return self._camera.ren_camera

    def spotlight(self):
        if not self._spotlight:
            self._spotlight = self.writer.global_value("spotlight")
        return self._spotlight.ren_spotlight

    def attach_stream(self, video_file, depth_file, scene):
        stream = self.writer.add_stream()    
        stream.video = video_file
        stream.depth = depth_file
        convert_or_fake_camera(stream.camera, scene)    



def convert_or_fake_camera(cam, scene):
    from camera import fake_camera
    if scene:
        convert_camera(cam, scene.camera)
    else:
        fake_camera(cam)

def convert_texture(txt, file, id):
    (txt.values, sz)  = util_image.read_rgb_image_as_str(file)
    txt.width = sz[0]
    txt.height = sz[1]
    txt.id = id


def generate_depth_stream_file(texture_file):
    import depth_stream    
    import os.path
    dest_path = os.path.join(config.data_folder, texture_file)
    depth_stream.generate_file(config.depth_images_folder, dest_path, config.depth_format)

def generate_dof_stream_file(texture_file):
    import depth_stream    
    import os.path
    dest_path = os.path.join(config.data_folder, texture_file)
    depth_stream.generate_file(config.dof_images_folder, dest_path, config.dof_format)


def get_scene_if_defined(index):
    import config
    from collada_loader import get_scene
    if config.collada_scene_file:
        if type(config.collada_scene_file) is list:
            return get_scene(config.collada_scene_file[index])
        else:
            return get_scene(config.collada_scene_file)
    else:
        return None

def acel_convert_assets(scene):     
    sra = SRAFile()
    if scene:         
        convert_geometry(scene, sra)     
        if scene.light:
            print "converting spotlight"
            convert_spotlight(sra.spotlight(), scene.light)

    convert_or_fake_camera(sra.camera(), scene)
 
    add_superpixels = False
    try:
        add_superpixels =  config.superpixels
    except:
        pass
    try:
        dof_stream = config.dof_stream
    except:
        dof_stream = None


    if type(config.video_texture_file) is list:
        for i, v in enumerate(config.video_texture_file):            
            sra.attach_stream(sra, v, config.depth_texture_file + str(i), get_scene_if_defined(i) )             
    else:
        depth_file = config.depth_texture_file         
        if config.export_depth:
            generate_depth_stream_file(depth_file)
        sra.attach_stream(config.video_texture_file,  depth_file, scene)        
        if add_superpixels:
            sra.attach_stream(config.mask_texture_file, depth_file, scene)
        if dof_stream:
            generate_dof_stream_file(dof_stream)
            sra.attach_stream(config.video_texture_file, dof_stream, scene)

    return sra

 


def process(filename):
    acel_convert_assets(get_scene_if_defined(0)).writer.write_to_file(filename)
    #print_file(filename)
    
    
if __name__ == "__main__":
    fname = config.result_file
    if len(sys.argv) > 1 :
        fname = sys.argv[1]    
    process(fname)
