import path_loader  # imports autogen python classes from scene-persistence
import config   # config files containing paths to data
import sys
from SRAWriter import SRAWriter
import surface_tracking_pb2

########################################################################

class Camera:
	fx = 0.0
	fy = 0.0
	cx = 0.0
	cy = 0.0
	width = 0
	height = 0
	R = []
	t = []
	
########################################################################

def ReadTokens(f):
	for line in f:
		for token in line.split():
			yield token
			
########################################################################

def LoadCameraFile(fname, cameras):
	f = open(fname, 'r')
	
	tokenGenerator = ReadTokens(f)
	
	numCam = int(tokenGenerator.next())
	model = int(tokenGenerator.next())
	
	for i in range(numCam):
		c = Camera()
		trow = int(tokenGenerator.next())
		brow = int(tokenGenerator.next())
		c.height = brow - trow + 1
		lcol = int(tokenGenerator.next())
		rcol = int(tokenGenerator.next())
		c.width = rcol - lcol + 1 
		
		c.fx = float(tokenGenerator.next())
		c.fy = float(tokenGenerator.next())
		c.cx = float(tokenGenerator.next())
		c.cy = float(tokenGenerator.next())
		c.R = []
		for j in range(9):
			c.R.append(float(tokenGenerator.next()))
		c.t = []
		for k in range(3):
			c.t.append(float(tokenGenerator.next()))
		
		cameras.append(c)
		
	f.close()
	
########################################################################	

def convertCamera(cam, camAcel) :
	ca = camAcel.Extensions[surface_tracking_pb2.st_camera]
	ca.fx = cam.fx
	ca.fy = cam.fy
	ca.cx = cam.cx
	ca.cy = cam.cy
	ca.width = cam.width
	ca.height = cam.height
	ca.R.extend(cam.R)  
	ca.t.extend(cam.t)  
	
########################################################################	

if __name__ == "__main__":
	
	
	if len(sys.argv) > 1:
		result_fname = sys.argv[1]    
	else:
		result_fname = "out.scene"
		
	cameras = []
	LoadCameraFile(config.camera_file, cameras)	
	
	# create property "cameras" which is an acel with a list of cameras as children 
	writer = SRAWriter()
	acelListCameras = writer.global_list("cameras")
	
	for i in range(len(cameras)):
		camAcel = acelListCameras.children.add()
		convertCamera(cameras[i], camAcel)
		
	scene = writer.get_scene()
	
	# test of field access
	print scene.globals.properties[0].value.children[1].Extensions[surface_tracking_pb2.st_camera].R[2]
		
	writer.write_to_file(result_fname)
    
	
	
