#import config   # config files containing paths to data - NO config used, specified through cmd arguments
import path_loader
import sys
import scenedb_pb2
import surface_tracking_pb2
from surface_tracking_pb2 import TCMesh
from SRAWriter import SRAWriter


class VertexTrack:
	id = 0
	frames = []  # frame numbers
	positions = [] # associated 3d positions - lists (x,y,z)

class MeshTopology:
	frame = 0 # frame number
	faces = [] # faces defined by vertex ids - lists (id0,id1,id2)

class MeshTrack:
	vertex_tracks = []  # list of vertexTrack objects
	face_history = [] # list of MeshTopology objects

########################################################################

def ReadTokens(f):
	for line in f:
		for token in line.split():
			yield token
			
########################################################################

def LoadMeshTrackFile(fname, meshtrack):
	fd = open(fname, 'r')
	
	#meshtrack = MeshTrack()
	tokens = ReadTokens(fd)
	
	# read vertex tracks
	num_tracks = int(tokens.next())
	meshtrack.vertex_tracks = []
	for i in range(num_tracks):
		vt = VertexTrack()
		vt.id = int(tokens.next())
		num_frames = int(tokens.next())
		vt.frames = []
		vt.positions = []
		for f in range(num_frames):
			vt.frames.append(int(tokens.next()))
			pos = []
			pos.append(float(tokens.next()))  # x
			pos.append(float(tokens.next()))  # y
			pos.append(float(tokens.next()))  # z
			vt.positions.append(pos)
					
			# skip orientation in the file
			float(tokens.next())
			float(tokens.next())
			float(tokens.next())
			
		meshtrack.vertex_tracks.append(vt)
			
	# read faces in individual frames
	num_frames_total = int(tokens.next())
	meshtrack.face_history = []
	for f in range(num_frames_total):
		top = MeshTopology()
		top.frame = int(tokens.next())
		num_faces = int(tokens.next())
		top.faces = []
		for tri in range(num_faces):
			ids = []
			ids.append(int(tokens.next()))
			ids.append(int(tokens.next()))
			ids.append(int(tokens.next()))
			top.faces.append(ids)
			
		meshtrack.face_history.append(top)
		
	fd.close()
	
########################################################################

def GetFrameRange(meshtrack):
	start_frame = meshtrack.face_history[0].frame
	end_frame = meshtrack.face_history[len(meshtrack.face_history)-1].frame
	return (start_frame, end_frame)
	
########################################################################

# returns (found status, position)
def FindFrame(face_history, frame):
	for i in range(len(face_history)):
		if(face_history[i].frame == frame):
			return (True, face_history[i].faces)
	
	return (False, [])

########################################################################

# returns (found status, position)
def FindPosition(vertex_track, frame):
	for i in range(len(vertex_track.frames)):
		if(vertex_track.frames[i] == frame):
			return (True, vertex_track.positions[i])
	
	return (False, [0,0,0])

########################################################################

# extract temporally consistent mesh	
#def GetMesh(meshtrack, frame):
	#tcmesh = TCMesh()
	
	#tcmesh.frame = frame
	
	#for v in range(len(meshtrack.vertex_tracks)):
		#tcmesh.vertex_ids.append(meshtrack.vertex_tracks[v].id)
		#status, pos = FindPosition(meshtrack.vertex_tracks[v], frame)
		#if(status):  # vertex exists in the frame
			#tcmesh.vertex_positions.extend(pos)
	
	#status, faces = FindFrame(meshtrack.face_history, frame)
	#if(status):  # vertex exists in the frame
		#for f in range(len(faces)):
			#tcmesh.faces.extend(faces[f])
	
	#return tcmesh
	
def GetMesh(meshtrack, frame, tcmesh):
		
	tcmesh.frame = frame
	
	for v in range(len(meshtrack.vertex_tracks)):
		status, pos = FindPosition(meshtrack.vertex_tracks[v], frame)
		if(status):  # vertex exists for the frame
			tcmesh.vertex_ids.append(meshtrack.vertex_tracks[v].id)
			tcmesh.vertex_positions.extend(pos)
	
	status, faces = FindFrame(meshtrack.face_history, frame)
	if(status):  # faces exist for the frame
		for f in range(len(faces)):
			tcmesh.faces.extend(faces[f])
	
		
########################################################################

if __name__ == "__main__":
	
	if len(sys.argv) != 3:
		print "parameters: <input mesh track file> <ouput scene file>"
		exit 
	
	track_fname = sys.argv[1]    
	result_fname = sys.argv[2]    
		
	meshtrack = MeshTrack()
	LoadMeshTrackFile(track_fname, meshtrack)
	
	# create property acel "meshes" with a list of frames as children
	writer = SRAWriter()
	al_frames = writer.global_list("meshes")
	
	start, end = GetFrameRange(meshtrack)
	for f in range(start, end+1):
		acel = al_frames.children.add()
		GetMesh(meshtrack, f, acel.Extensions[surface_tracking_pb2.st_tcmesh])
	
	scene = writer.get_scene()	
			
	print len(scene.globals.properties[0].value.children[1].Extensions[surface_tracking_pb2.st_tcmesh].faces)/3
	
	writer.write_to_file(result_fname)
		
	
	
