import path_loader
import sys
from depth_stream_pb2 import Data
import numpy as np

   
#read dir and perform sort of file. Input arguments must be directory in the format: "C:\Python27"
# and extention of the files must be in the format: ".exr" or ".log" etc.    

def read_sorted_file_list(dir, format):
    import os
    for file in sorted(os.listdir(dir)):
        if format.has_valid_extension(file):
            yield os.path.join(dir, file)

def generate_depth_stream(folder, file_format): 
    proto  = Data();
    files =[f for f in read_sorted_file_list(folder, file_format)]
    if len(files) == 0:
        print ("no depth files found, matching criteria: check the directory path and format")

    for file in files:
        data, size = file_format.read_depth(file)
        if np.min(data) < 0:
            print("warning: the depth values are negative, probably wrong representation") 
        proto.width  = size[0]
        proto.height = size[1]
        frame = proto.depth_stream.add()
        #print("data")
        #print(data)
        frame.values.extend([v for v in data])

    return proto
    
    
def write_to_file(filename, sc):
    f = open(filename, "wb")
    f.write(sc.SerializeToString())
    f.close()    

def generate_file(folder, filename, file_format):    
    data = generate_depth_stream(folder, file_format)
    write_to_file(filename, data)


def read_file_frame(filename):
    with open(filename, "rb") as f:
        d = Data() 
        d.ParseFromString(f.read())
        print (d.width, d.height)
        frames = [f for f in d.depth_stream]
        frame = frames[0]
        import numpy as np
        u = np.array(frame.values)
        import scipy.io as sio
        sio.savemat("frame.mat", {"frame":u.reshape((d.height, d.width))})



