import numpy as np
from util_image import *

inverted_depth_in_tiff = True
depth_channel_name = ['Z', 'Y']

#convert depth from the range 0 - 2**16 to meters
def uint16_tiff_to_float_depth_transform(depth):
    range = 10.0
    min_range = 2.0 

    #print np.min(depth), np.max(depth)
    x = np.float64(depth) / float(2**16 - 1)

    if inverted_depth_in_tiff:
      x = np.ones(x.shape) - x
    return x * range + min_range

def read_depth_uint16_tiff_as_float(name):
    data, size  = read_uint16_tiff(name)    
    floats = uint16_tiff_to_float_depth_transform(data)
    return floats, size

def write_depth_to_exr(name, size, data):
    write_float_channel_to_exr(name, depth_channel_name[0], size, data)  

def read_depth_from_exr(name):
    u = None
    for dc in depth_channel_name:     
        try:
            print(dc)
            return read_exr_channel_as_array(name, dc)
        except TypeError as e :
            u = e
    raise u

def convert_depth_uint16_tiff_to_exr(src, dst):     
    data, size = read_depth_uint16_tiff_as_float(src)
    write_depth_to_exr(dst, size, data)
 

def is_tiff_file(file):
    return file.endswith(".tiff") or file.endswith(".tif")

def is_exr_file(file):
    return file.endswith(".exr")


#depth formats
class depth_format_uint16_tiff_with_rescaling:
    read_depth = staticmethod(read_depth_uint16_tiff_as_float)
    has_valid_extension = staticmethod(is_tiff_file)    
class depth_format_float_tiff:
    read_depth =staticmethod( read_float_tiff )
    has_valid_extension = staticmethod(is_tiff_file)
class depth_format_exr:
    read_depth = staticmethod(read_depth_from_exr)
    has_valid_extension = staticmethod(is_exr_file)
    


#when run as a script converts uint16tiff to an exr
if __name__ == "__main__":
    from sys import argv
    name = argv[1]
    d1, s1 = read_depth_uint16_tiff_as_float(name)
    #write_float_as_rgb_image(name + ".png", d1, s1)
    write_matlab_matrix("depth" ,d1)
    convert_depth_uint16_tiff_to_exr(name, name + ".exr")    
