import os
import sys
from PIL import Image
import numpy as np
 

def read_uint16_tiff(image_name):
    assert(image_name.endswith(".tif") or image_name.endswith(".tiff"))
    im = Image.open(image_name)
    data = np.array(im.getdata(), dtype="uint16")         
    return data, im.size

def write_float_channel_to_exr(fname, channel_name, size, data):
    import OpenEXR, Imath
    header =  OpenEXR.Header(size[0], size[1])
    header['channels'] = {channel_name : Imath.Channel(Imath.PixelType(OpenEXR.FLOAT))}
    out = OpenEXR.OutputFile(fname, header)     
    out.writePixels({channel_name : np.float32(data).tostring()})

def write_float_tiff(name, data, size):
    assert(name.endswith(".tiff") or name.endswith(".tif"))
    im =  Image.fromarray(np.float32(data.reshape((size[1], size[0]))))
    im.save(name)

def read_float_tiff(name):
    assert(name.endswith(".tiff") or name.endswith(".tif"))    
    im = Image.open(name)
    return [v for v in im.getdata()], im.size

def read_exr_channel_as_str(exrfile, channel_name):
    assert(exrfile.endswith(".exr") )
    import OpenEXR
    import Imath
    fhandle = OpenEXR.InputFile(exrfile)
    dw = fhandle.header()['dataWindow']
    size = (dw.max.x - dw.min.x + 1, dw.max.y - dw.min.y + 1)
    zstr = fhandle.channel(channel_name, Imath.PixelType(Imath.PixelType.FLOAT))
    return zstr, size

def read_exr_channel_as_array(exrfile, channel_name):
    d, s = read_exr_channel_as_str(exrfile, channel_name)
    return  np.float64(np.fromstring(d, 'float32')), s

def read_rgb_image_as_float_gray(fname):
    im = Image.open(fname).convert("L")        
    p = np.array(im.getdata(), dtype="float32")
    return p / 255, im.size

# doesn' work due to transposition
#def read_rgb_image_as_float_hue(fname):
#    from  skimage.data import imread
#    from skimage.color import rgb2hsv
#    im = imread(fname)
#    im = rgb2hsv(im)[:, :, 0]
#    #print im
#    #p = np.array(im.getdata(), dtype="float32")    
#    return np.ravel(im), im.shape


def read_rgb_image_as_str(fname):
    im = Image.open(fname).convert("RGB")    
    pixels = [c for t in im.getdata() for c in t]    
    return "".join(map(chr, pixels)), im.size

def convert_exr_channel_to_float_tiff(exrfile, channel_name):    
    zstr, size = read_exr_channel_as_str(exrfile, channel_name)
    im = Image.fromstring('F', size, zstr)
    im.save(exrfile + ".tiff")    



#this is only for debug:
def convert_float_matrix_to_rgb_image(mat):
    mn = np.min(mat) 
    mx = np.max(mat)
    r = mx - mn
    u = np.ones(mat.shape)
    o = np.zeros(mat.shape)
    if r != 0:
        qq = (mat - mn*u )/ r
    else:
        qq = np.maximum(np.minimum(mat, u), o)
    im =  Image.fromarray(np.round(255*qq))
    return im.convert("RGB")
     
def write_float_as_rgb_image(name, data, size):
    mat = data.reshape((size[1], size[0]))    
    write_float_matrix_as_rgb_image(name, mat)    

def write_float_matrix_as_rgb_image(name, mat):    
    im = convert_float_matrix_to_rgb_image(mat)
    im.convert("RGB").save(name)


def write_matlab_matrix(name, data):
    import scipy.io
    scipy.io.savemat(name + '.mat', mdict={name: data})  

def read_matlab_matrix(name, fname ):
    import scipy.io    
    d = scipy.io.loadmat(fname + ".mat" )
    return  d[name]



#write_float_matrix_as_rgb_image("bg.png", read_matlab_matrix("bg_st", "bg_st"))