/*
 * protoMesh.cpp
 *
 *  Created on: Mar 14, 2014
 *      Author: christopher
 */

#include "backendMesh.h"

//bool importMesh( const std::string& pFile) {
int setMesh(PAcel parent, string filename, IdManager::IdManager &idM, meshType::type type, int **ids) {
	int numOfMeshes;
	if(type < 38) { // all mesh types with enums < 38 are supported by ASSIMP
		numOfMeshes = setMeshAssimp(parent, idM, filename, type, ids);
	}
	//else if(type < ) {
		// other mesh types using other libraries ????
	//}
	else {
		cerr << "Unknown type.\n" << endl;
		return 0;
	}

	//cout << "Mesh \"" << filename << "\" added to Scene." << endl;
	return numOfMeshes;
}

namespace backend {

aiMesh *get_mesh(PAcel mesh){
	aiMesh *myMesh = new aiMesh;
	if(mesh->Property("vertices")) {
		int numOfVertices = mesh->Property("vertices")->List()->Count();
		//cout << "num of vertices found is " << numOfVertices << endl;
		myMesh->mNumVertices = numOfVertices;
		myMesh->mVertices = new aiVector3D[numOfVertices];
		for (int i=0; i<numOfVertices; i++) {
			myMesh->mVertices[i].Set(mesh->Property("vertices")->List()->Get(i)->Value()->GetFloat3().x,mesh->Property("vertices")->List()->Get(i)->Value()->GetFloat3().y,mesh->Property("vertices")->List()->Get(i)->Value()->GetFloat3().z);
		}
	}
	if(mesh->Property("faces")) {
		int numOfFaces = mesh->Property("faces")->List()->Count();
		myMesh->mNumFaces = numOfFaces;
		myMesh->mFaces = new aiFace[numOfFaces];
		for(int i=0; i<numOfFaces; i++) {
			int numOfVertices = mesh->Property("faces")->List()->Get(i)->Property("numOfVertices")->Value()->GetInt();
			myMesh->mFaces[i].mNumIndices = numOfVertices;
			myMesh->mFaces[i].mIndices = new unsigned int[numOfVertices];
			for (int j=0; j<numOfVertices; j++) {
				myMesh->mFaces[i].mIndices[j] = mesh->Property("faces")->List()->Get(i)->Property("verticeByIndex")->List()->Get(j)->Value()->GetInt();
			}
		}
	}
	return myMesh;
}

}


/*
 * use ASSIMP Library for the following formats:
 * Collada ( *.dae;*.xml ), Blender ( *.blend ), Biovision BVH ( *.bvh ), 3D Studio Max 3DS ( *.3ds ),3D Studio Max ASE ( *.ase ), Wavefront Object ( *.obj ), Stanford Polygon Library ( *.ply ), AutoCAD DXF ( *.dxf ), IFC-STEP, Industry Foundation Classes ( *.ifc ),
 * Neutral File Format ( *.nff ), Sense8 WorldToolkit ( *.nff ), Valve Model ( *.smd,*.vta ), Quake I ( *.mdl ), Quake II ( *.md2 ), Quake III ( *.md3 ), Quake 3 BSP ( *.pk3 ), RtCW ( *.mdc ), Doom 3 ( *.md5mesh;*.md5anim;*.md5camera ), DirectX X ( *.x ), Quick3D ( *.q3o;q3s ), Raw Triangles ( .raw ), AC3D ( *.ac ), Stereolithography ( *.stl ), Autodesk DXF ( *.dxf ), Irrlicht Mesh ( *.irrmesh;*.xml ),
 * Irrlicht Scene ( *.irr;*.xml ), Object File Format ( *.off ), Terragen Terrain ( *.ter ), 3D GameStudio Model ( *.mdl ), 3D GameStudio Terrain ( *.hmp ), Ogre (*.mesh.xml, *.skeleton.xml, *.material), Milkshape 3D ( *.ms3d ),
 * LightWave Model ( *.lwo ), LightWave Scene ( *.lws ), Modo Model ( *.lxo ), CharacterStudio Motion ( *.csm ), Stanford Ply ( *.ply ), TrueSpace ( *.cob, *.scn ), XGL ( *.xgl, *.zgl )
 */
int setMeshAssimp(PAcel parent, IdManager::IdManager &idM, string filename, meshType::type type, int **ids) {

	// Create an instance of the Importer class
	Assimp::Importer importer;

	unsigned int numOfMeshes = 0; // return value: number of meshes in file

	// And have it read the given file with some example postprocessing
	// Usually - if speed is not the most important aspect for you - you'll
	// propably to request more postprocessing than we do in this example.
	const aiScene* scene = importer.ReadFile( filename,
			aiProcess_CalcTangentSpace |
			aiProcess_Triangulate |
			aiProcess_JoinIdenticalVertices |
			aiProcess_SortByPType);

	// If the import failed, report it
	if( !scene) {
		cerr << importer.GetErrorString();
		return 0;
	}

	// else check if scene contains mesh
	if(scene->HasMeshes()) {
		numOfMeshes = scene->mNumMeshes;
		*ids = new int[numOfMeshes];// allocate memory for IDs
		//bool hasNormals = false;
		//bool hasTangents = false;
		//bool hasBitangents = false;
		for(unsigned int i=0; i<numOfMeshes; i++) {
			int id = idM.AllocateId();
			//cout << "set mesh id " << id <<  endl;
			*(*ids+i) = id;
			aiMesh* input = scene->mMeshes[i];	// get mesh from input
			PAcel mesh = parent->AddChild(id);	// add mesh to base layer
			//cout << "added mesh to scene" << endl;
			if("" != (string)input->mName.data) {
				PAcel name = mesh->AddProperty("name",idM.AllocateId());
				name->SetValue()->SetString((string)input->mName.data);
			}
			PAcel vertices = mesh->AddProperty("vertices",idM.AllocateId());
			PAcel faces, bones, normals, texture, color, positions, tangents, bitangents;
			if(input->HasFaces()) {
				int id2 = idM.AllocateId();
				faces = mesh->AddProperty("faces",id2);
				//cout << "id of faces is " << id2 << endl;
				//cout << "set id is " << mesh->Property("faces")->Id() << endl;
				//faces = mesh->AddProperty("faces",idM.AllocateId());
			}
			if(input->HasBones()) {
				bones = mesh->AddProperty("bones",idM.AllocateId());
			}
//			if(input->HasNormals()) {
//				normals = mesh->AddProperty("normals",idM.AllocateId());
//				hasNormals = true;
//			}
			if(input->HasPositions()) {
				positions = mesh->AddProperty("positions",idM.AllocateId());
			}
			if(input->HasTextureCoords(1)) {
				texture = mesh->AddProperty("texture",idM.AllocateId());
			}
//			if(input->HasTangentsAndBitangents()) {
//				tangents = mesh->AddProperty("tangents",idM.AllocateId());
//				bitangents = mesh->AddProperty("bitangents",idM.AllocateId());
//				hasTangents = true;
//				hasBitangents = true;
//			}
			if(input->HasVertexColors(1)) {
				color = mesh->AddProperty("color",idM.AllocateId());
			}
			for(unsigned int j=0; j<input->mNumVertices; j++) {	// add vertices to mesh
				PAcel vert = vertices->AddChild(idM.AllocateId()); // each vertice is new acel
				aiVector3D vertice = input->mVertices[j];
				vert->SetValue()->SetFloat3(sra::float3_t(vertice.x, vertice.y, vertice.z));
			}

			for(unsigned int j=0; j<input->mNumFaces; j++) { // add faces to mesh
				aiFace face = input->mFaces[j];
				int id1 = idM.AllocateId();
				PAcel fac = faces->AddChild(id1);
				//PAcel fac = faces->AddChild(idM.AllocateId());
				//cout << "Acel with id " << id1 << " has property _numOfVertices_" << endl;
				//cout << "set ID is " << fac->Id() << endl;
				PAcel numV = fac->AddProperty("numOfVertices",idM.AllocateId());
				numV->SetValue()->SetInt(face.mNumIndices);

				PAcel v_ind = fac->AddProperty("verticeByIndex",idM.AllocateId());
				PAcel v_ids = fac->AddProperty("verticeByIDs",idM.AllocateId());

				for(unsigned int k=0; k<face.mNumIndices; k++) {
					PAcel index = v_ind->AddChild(idM.AllocateId());
					index->SetValue()->SetInt(face.mIndices[k]);
					PAcel id = v_ids->AddChild(idM.AllocateId());
					id->SetValue()->SetInt(vertices->List()->Get(face.mIndices[k])->Id());
				}
			}
		}
	}

	return numOfMeshes;
}
