/*
 * sraProto.h
 *
 *  Created on: Mar 4, 2014
 *      Author: kelvinchelli
 */

#ifndef SRAPROTO_H_
#define SRAPROTO_H_
#include <string>
#include <iostream>     // std::cout
#include <fstream>      // std::ifstream
#include <sstream>      // stringstream
#include <new>

#include <scene-core/sra-api/sra_scene.h>

#include "IdManager.h"

#include "filetypes.h"

#include "backendMap.h"
#include "backendMesh.h"

namespace backend {

int set_Map(PScene scene, IdManager::IdManager &idm, std::string filename, mapType::type type, int parentId);

int set_Mesh(PScene scene, IdManager::IdManager &idM, std::string filename, meshType::type type, int parentId, int **ids);

PAcel getAcel(PAcel parent, int id);

int setFilePath(PScene scene, IdManager::IdManager &idM, int parentId, string path);
string getFilePath(PScene scene, int id);

int setBinaryFile(PScene scene, IdManager::IdManager &idm, int parentId, string path);
int getBinaryFile(PScene scene, int id, char **buffer);

/**
 * Create an acel containing the camera projection matrix and some identification information. The acel will be a
 * to the acel with the id parentId.
 *
 * @param[in]  scene                  Whole scene graph containing the parent node for the new camera acel.
 * @param[in]  idm                    IdManager to generate new Ids. parentId should already be marked as used.
 * @param[in]  cameraName             Name of the camera. This can be any type of string. Empty strings are not allowed.
 * @param[in]  projectionMatrix       3x4 projection matrix for this camera. It is recommended to store the values row-wise.
 * @param[in]  correspondingStreams   List of names of streams this camera is used for. This parameter is optional
 * @param[in]  parentId               Id of the parent acel where the new camera acel will be attached to.
 *
 * @throws InvalidAcelError  The given scene graph is invalid.
 *
 * @return Id of the camera acel that was created.
 */
int addCameraProjectionMatrix(PScene scene,
                              IdManager::IdManager& idm,
                              const std::string cameraName,
                              const sra::float_buffer_t& projectionMatrix,
                              const std::vector<std::string>& correspondingStreams,
                              const int parentId);

/**
 * Parse an acel containing the camera projection matrix and some identification information. The up to three properties
 * of the camera acel are expected to be direct members of the given node.
 *
 * @param[in]  scene                Whole scene graph containing the camera acel.
 * @param[in]  id                   Id of the parent acel of the camera acel. This is the return value of addCameraProjectionMatrix().
 * @param[out] cameraName           Name that was found in the NAME property of the parent acel.
 * @param[out] projectionMatrix     Projection matrix that was found in the MATRIX property of the parent acel.
 * @param[out] correspondingStreams List of names of streams this camera is used for. Found in the STREAM property of the
 *                                  parent acel. May be empty.
 *
 * @throws InvalidAcelError  The given scene graph is invalid.
 *
 * @return True == Node was parsed successfully.
 *         False == Parsing of the node failed. Using the return values will result in undefined behavior.
 */
bool getCameraProjectionMatrix(const PScene scene,
                               const int id,
                               std::string& cameraName,
                               sra::float_buffer_t& projectionMatrix,
                               std::vector<std::string>& correspondingStreams);

void blockIds(PScene scene, IdManager::IdManager &idM);
void blockChildIds(PAcel acel, IdManager::IdManager &idM);

bool remove_Acel(PScene scene, IdManager::IdManager &idm, int id);
int removeChildAcel(PAcel acel, int id);

} /* namespace proto */

#endif /* SRAPROTO_H_ */
