/*
 * LoadTCMeshesSRA.cpp
 *
 *  Created on: 18 Apr 2014
 *      Author: mk0024
 *  $Id: LoadTCMeshesSRA.cpp 476 2014-05-13 16:04:26Z mklaudiny $
 */

#include <string>
#include <cstdio>
#include <fstream>

#include "scene-core/sra-api/sra_scene.h"
#include "scene-core/sra_implementation.h"
#include "scene-core/XScene.h"

#include "boost/program_options/options_description.hpp"
#include "boost/program_options/parsers.hpp"
#include "boost/program_options/variables_map.hpp"
namespace po = boost::program_options;

#include "sra_tc_meshes.h"

/*****************************************************************************/

int main(int argc, char *argv[])
{

	std::string sraFilename;

	try {
		po::options_description desc("Options");
		desc.add_options()
			("help", "Help message")
			("input,i", po::value<std::string>(&sraFilename)->default_value("tc_meshes.scene"), "SRA/SCENE file. [input]")
			;

		po::variables_map vm;
		po::store(po::parse_command_line(argc, argv, desc), vm);
		po::notify(vm);

		if (vm.count("help")) {
			std::cout << "Load scene file with temporally consistent mesh sequence." << std::endl;
			std::cout << desc << std::endl;
			return 1;
		}

		// print commandline
		for(int j = 0; j < argc; ++j) { std::cout << argv[j] << " "; }
		std::cout << std::endl;
	}
    catch(std::exception& e)
    {
        std::cerr << e.what() << std::endl;
        return -1;
    }

    PScene scene;

    try {
		scene = sra::get_scene_representation(sraFilename);
	}
	catch(std::invalid_argument& e) {
		std::cout << "Cannot load the scene file: " << e.what() << std::endl;
	}

	// std::cout << ((XScene*) scene.get())->GetPBScene()->DebugString();

	// get the mesh sequence acel
	PAcel acelSeq = scene->GetGlobals()->List()->Get(0);

	try {

		std::cout << "Name: " << acelSeq->Property(TCMESHES_NAME)->Value()->GetString() << std::endl;
		std::string path = acelSeq->Property(TCMESHES_PATH)->Value()->GetString();
		std::cout << "Mesh path and name: " << path << std::endl;

		sra::IAcelList* children = acelSeq->List();
		for(int i = 0; i < children->Count(); ++i) {
			unsigned int frame = children->Get(i)->Property(TCMESHES_FRAME)->Value()->GetInt();
			std::cout << "Frame: " <<  frame << std::endl;

			char file[300];
			sprintf(file, path.c_str(), frame);
			std::cout << "Mesh: " << file << std::endl;

			// NOTE: access to int array  ids.Ptr()
			sra::int_buffer_t ids = children->Get(i)->Property(TCMESHES_INDEX2ID)->Value()->GetIntBuffer();
			std::cout << "Index to ids: size = " <<  ids.Size() << std::endl;
			for(unsigned int j = 0; j < ids.Size(); ++j) {
				std::cout << ids.Ptr()[j] << " ";
			}
			std::cout << std::endl;

			// NOTE: the vertex on the index 5 in OBJ file has the following id:  ids.Ptr()[5]

		}

	}
	catch(sra::UnsupportedPropertyError& e) {
		std::cout << "Accessing undefined property." << std::endl;
	}

	return 0;
}
