/*
 * sra.cpp
 *
 */

#include "sra.h"
using namespace std;

Scene::Scene(string sceneFile) {

	this->sceneFile = sceneFile; // set scene file name
	this->idM.MarkAsUsed(0); // block 0 as well

	// initialize SCENE file
    fstream input(sceneFile.c_str(), ios::in | ios::binary);
    if (!input) {
      cout << sceneFile << ": File not found.  Creating a new file." << endl;
      this->scene = sra::make_scene_representation(".");
      this->idM.MarkAsUsed(1); // id 1 is reserved for globals
    } else {
        this->scene = sra::get_scene_representation(sceneFile);
        // block already existing id's in scene file
        backend::blockIds(this->scene, this->idM);
    }

}

Scene::~Scene() {
	sra::set_scene_representation(this->sceneFile, this->scene);
}

int Scene::writeScene(string sceneFile){
	this->sceneFile = sceneFile;

	sra::set_scene_representation(this->sceneFile, this->scene);

	return 0;
}

bool Scene::removeAcel(int id){
	return backend::remove_Acel(this->scene,this->idM,id);
}

int Scene::addMap(string filename, mapType::type type, int parentId){
	return backend::set_Map(this->scene,this->idM,filename,type,parentId);
}

CImg<unsigned char> *Scene::getMap(int id){
	return backend::get_map(backend::getAcel(this->scene->GetGlobals(),id));
}

int Scene::addMesh(string filename, meshType::type type, int parentId, int **ids){
	return backend::set_Mesh(this->scene,this->idM,filename,type,parentId,ids);
}

aiMesh *Scene::getMesh(int id) {
	return backend::get_mesh(backend::getAcel(this->scene->GetGlobals(),id));
}


int Scene::addFilePath(string filename, int parentId){
	return backend::setFilePath(this->scene,this->idM,parentId,filename);
}

string Scene::getFilePath(int id){
	return backend::getFilePath(this->scene,id);
}

int Scene::addCameraProjectionMatrix(const std::string cameraName,
                                     const sra::float_buffer_t &projectionMatrix,
                                     const std::vector<std::string> &correspondingStreams,
                                     const int parentId)
{
	return backend::addCameraProjectionMatrix(this->scene, this->idM, cameraName, projectionMatrix, correspondingStreams, parentId);
}

bool Scene::getCameraProjectionMatrix(const int id,
                                      std::string &cameraName,
                                      sra::float_buffer_t &projectionMatrix,
                                      std::vector<std::string> &correspondingStreams) const
{
	return backend::getCameraProjectionMatrix(this->scene, id, cameraName, projectionMatrix, correspondingStreams);
}

int Scene::addBinaryData(string filename, int parentId) {
	return backend::setBinaryFile(this->scene,this->idM,parentId,filename);
}

int Scene::getBinaryData(int id, char **buffer) {
	return backend::getBinaryFile(this->scene,id,buffer);
}

PScene Scene::getScene() {
	return this->scene;
}
