#ifndef EXR_FRAMES_TEXTURE_LOADER_H
#define EXR_FRAMES_TEXTURE_LOADER_H

#include "stdint.h"
#include <string>
#include <vector>
#include "texture_loader_api.h"

namespace cv
{
  class Mat;
}

// Reads a sequence of exr files using openexr library 
class EXRTextureLoader : public ITextureLoader
{
  private:

    std::string frames_name;
    std::vector<std::string> channel_names;
    cv::Mat    *frame;
    int   first_frame;
    int    last_frame;
    int frame_step;

    // tries to set current_frame, width, height, channels, depth and
    // channel_names
    void Initialize();

  public:

    /* We expect a filename with the following printf format
     * [path/prefix]%0nd[suffix].exr
     *
     * Fields in [] are optional. Nor the path/prefix neither
     * postfix may have a '%' character. */
    EXRTextureLoader(std::string file, int first_frame = -1, 
      int last_frame = -1, int frame_step = -1, 
      std::vector<std::string> ch_names = std::vector<std::string>(0)); 
    EXRTextureLoader(pb::Stream & s, std::string work_dir); 

    virtual ~EXRTextureLoader();

    virtual bool NextFrame();
    virtual void SetCurrentData(const uint8_t* data, int w=-1, int h=-1, int c=-1, int d=-1);
};

#endif

