#include "frame_seq_utils.h"
#include <fstream>
#include <cmath>

inline bool exists(const std::string& name)
{
  std::ifstream f(name.c_str());
  bool good = f.good();
  f.close();
  return good;
}

void DiscoverFrameSeqNumbering(const std::string frames_name, 
		int &first_frame, int &frame_step, int &last_frame)
{
  // parse frames_name to determine number of digits %0nd
  size_t i0 = frames_name.find('%');
  size_t i1 = frames_name.find('d',i0++);
  int n = std::stoi(frames_name.substr(i0,i1-i0));

  // set first_frame from files
  if (first_frame == -1)
  {
    // maximum frame index with the given digits
    int max_n = std::pow(10.0,n);

    // search for first frame with specified format
    char * name = new char[frames_name.size() + 10];
    do
    {
      first_frame += 1;
      std::sprintf(name,frames_name.c_str(),first_frame);
    }
    while (!exists(name) && first_frame < max_n);
    delete [] name;
  }

  // set frame_step from files
  if (frame_step == -1)
  {
    // maximum frame index with the given digits
    int max_n = std::pow(10.0,n);

    // search for first frame with specified format
    char * name = new char[frames_name.size() + 10];
    frame_step = 0;
    do
    {
      frame_step ++;
      std::sprintf(name,frames_name.c_str(),first_frame + frame_step);
    }
    while (!exists(name) && first_frame + frame_step < max_n);
    delete [] name;
  }

  // set last_frame from files
  if (last_frame <= first_frame)
  {
    // maximum frame index with the given digits
    int max_n = std::pow(10.0,n);

    // search for last frame with specified format
    char * name = new char[frames_name.size() + 10];
    last_frame = first_frame;
    do
    {
      last_frame += frame_step;
      std::sprintf(name,frames_name.c_str(),last_frame);
    }
    while (exists(name) && last_frame <= max_n);
    delete [] name;
  }
}

