#ifndef OFLOW_TEXTURE_LOADER_H
#define OFLOW_TEXTURE_LOADER_H

#include "stdint.h"
#include <string>
#include "texture_loader_api.h"

namespace cv
{
	class Mat;
}

// reads a sequence of oflow frames and stores it in opencv Mat images
class OFlowTextureLoader : public ITextureLoader
{
	private:

		std::string flows_path;
		cv::Mat    *flow;
		cv::Mat    *flow_components;
		int   first_frame;
		int    last_frame;
		int frame_step;

		void Initialize(const std::string filename);

	public:

		/* For each frame, we expect four filenames with the following printf
		 * format:
		 * [path/prefix]%0nd_u.oflow
		 * [path/prefix]%0nd_v.oflow
		 *
		 * Any of these can be given as file_name input. Fields in [] are
		 * optional. The path/prefix mustn't have a '%' character. 
		 *
		 * The texture loader outputs a floating point image with two channels,
		 * for the u and v components respectivelly. */
		OFlowTextureLoader(std::string file_name, int first_frame = -1, 
				int last_frame = -1, int frame_step = -1); 
		OFlowTextureLoader(pb::Stream &s, std::string work_dir); 
		virtual ~OFlowTextureLoader();

		virtual bool NextFrame();
		virtual void SetCurrentData(const uint8_t* data, int w=-1, int h=-1, int c=-1, int d=-1){};
};

#endif

