#include "texture_loader_api.h"
#include "video_texture_loader.h"
#include "frames_texture_loader.h"
#include "exr_frames_texture_loader.h"
#include "oflow_texture_loader.h"

#include <scene-persistence/autogen/scenedb.pb.h>
#include <scene-persistence/autogen/stream_fsio.pb.h>

#include <iostream>
#include <stdexcept>


std::shared_ptr<ITextureLoader> ITextureLoader::CreateTextureLoader(std::string file,
		int firstf, int lastf, int fstep, std::vector<std::string> channel_names)
{
	// get file extension
	size_t i = file.rfind('.');
	std::string extension = file.substr(i);

	if (channel_names.size() && extension != ".exr")
		throw std::logic_error("channels names have been specified for a non-EXR image.");

	// decide with texture loader to instantiate
	if (extension == ".avi" || extension == ".mpg" || extension == ".mp4" || extension == ".mov") 
		return std::shared_ptr<ITextureLoader>(new  VideoTextureLoader(file));
	if (extension == ".oflow") return std::shared_ptr<ITextureLoader>(new  OFlowTextureLoader(file,firstf,lastf,fstep));
	if (extension == ".png")   return std::shared_ptr<ITextureLoader>(new FramesTextureLoader(file,firstf,lastf,fstep));
	if (extension == ".exr")  	return std::shared_ptr<ITextureLoader>(new    EXRTextureLoader(file,firstf,lastf,fstep,channel_names));

	// no loader for texture format
	return std::shared_ptr<ITextureLoader>();
}

std::shared_ptr<ITextureLoader> ITextureLoader::CreateTextureLoader(pb::Stream & s, std::string work_dir)
{
	// we keep the TexLoader uninitialized if there is no video name
	if (!s.has_video()) return nullptr;

	// get file extension
	std::string file = s.video();
	size_t i = file.rfind('.');
	std::string extension = file.substr(i);

//	std::cout << "\t\tITextureLoader::CreateTextureLoader -> creating " << file << " extension " <<  extension << "\n";


	// decide with texture loader to instantiate
	if (extension == ".avi" || extension == ".mpg" || extension == ".mp4" || extension == ".mov")
		return std::shared_ptr<ITextureLoader>(new  VideoTextureLoader(s, work_dir));
	if (extension == ".oflow") return std::shared_ptr<ITextureLoader>(new  OFlowTextureLoader(s, work_dir));
	if (extension == ".png")   return std::shared_ptr<ITextureLoader>(new FramesTextureLoader(s, work_dir));
	if (extension == ".exr")   return std::shared_ptr<ITextureLoader>(new    EXRTextureLoader(s, work_dir));

	// no loader for texture format
	return std::shared_ptr<ITextureLoader>();
}
