#include "video_texture_loader.h"
#include <stdexcept>
#include <iostream>

#include <scene-persistence/autogen/scenedb.pb.h>
#include <scene-persistence/autogen/stream_fsio.pb.h>

const size_t RGB_SIZE = 3;

void VideoTextureLoader::Load(std::string filename)
{
  if (avbin_init()) 
    throw new std::runtime_error("error initializing avbin");

  file = avbin_open_filename(filename.c_str());
  if (!file)   
  {
    std::cout << " could not open video file " << filename;
    throw std::invalid_argument(filename);
  }
   
  AVbinFileInfo fileinfo;
  fileinfo.structure_size = sizeof(fileinfo);

  if (avbin_file_info(file, &fileinfo)) 
    throw new std::runtime_error("invalid file info");

  video_stream = 0;
  video_stream_index = -1;

  int stream_index;
  for (stream_index=0; stream_index<fileinfo.n_streams; stream_index++)
  {
    AVbinStreamInfo streaminfo;
    streaminfo.structure_size = sizeof(streaminfo);

    avbin_stream_info(file, stream_index, (AVbinStreamInfo *) &streaminfo);

    if (streaminfo.type == AVBIN_STREAM_TYPE_VIDEO)
    {
      width = streaminfo.video.width;
      height = streaminfo.video.height;
		channels = RGB_SIZE; // FIXME: only handles videos with 3 channels
		depth = 1;           // FIXME: only handles videos with 8 bits per color
      video_stream_index = stream_index;
      video_stream = avbin_open_stream(file, stream_index);

      video_buffer = (uint8_t*) malloc(width*height*RGB_SIZE);
      if (video_stream) return;
    }
  }
  throw new std::runtime_error("no video stream");
}


bool VideoTextureLoader::NextFrame()
{
  if (!file) return false;
//int cnt = 0;

  AVbinPacket packet;
  packet.structure_size = sizeof(packet);
  while (!avbin_read(file, &packet))
  {
    if ((int)packet.stream_index == video_stream_index)
    {
      if (avbin_decode_video(video_stream, packet.data, packet.size,video_buffer) <=0)
        continue;
      return true;
    }
  }
  return false;
}

VideoTextureLoader::~VideoTextureLoader() 
{
  if (video_stream) avbin_close_stream(video_stream);
  if (file) avbin_close_file(file);
  if (video_buffer) free(video_buffer);
}


VideoTextureLoader::VideoTextureLoader( std::string filename ) : 
  video_stream(0), file(0), video_stream_index(-1)
{
  Load(filename);
}

VideoTextureLoader::VideoTextureLoader( pb::Stream &s, std::string work_dir ) :
  video_stream(0), file(0), video_stream_index(-1)
{
	Load(work_dir + s.video());
	stream = &s;
}
