/*
 * test.cpp
 *
 *  Created on: Jan 27, 2014
 *      Author: christopher
 */

#include <string>
#include <fstream>
#include <scene-persistence/sra.h>
#include <scene-persistence/backend/filetypes.h>

using namespace std;

/* source file to test the sra */
int main() {
	/**
	 * Test Scene initialization
	 */
	string sceneName = "myScene.sra";

	Scene myScene(sceneName);

	string imageName = "test.pgm";


	int id1, id2; // id of new acels written to Scene

	/**
	 * Test writing file path to scene
	 */
	cout << "---------------- File Path Test --------------------" << endl;

	id1 = myScene.addFilePath(imageName,2);

	cout << "new string has id " << id1 << endl;

	/**
	 * test reading acel property "path_to_file" from scene
	 */
	cout << "path to file from acel " << id1 << " is " << myScene.getFilePath(id1) << endl;
	cout << "-------------------------------------------------------" << endl << endl;


	/**
	 * Test writing binary data to Scene
	 */
	cout << "--------------- Binary Data Test ------------------" << endl;

	string meshName = "cube.obj";

	id2 = myScene.addBinaryData(meshName,2);

	cout << "new binary buffer has id " << id2 << endl;


	/**
	 * test reading acel property "binary_data" from file
	 */
	std::ofstream outfile ("new.obj",std::ofstream::binary);
	char * buffer;
	int size = myScene.getBinaryData(id2, &buffer);
	cout << "length of buffer from acel " << id2 << " is " << size << endl;
	outfile.write(buffer,size);
	outfile.close();
	cout << "-------------------------------------------------------" << endl << endl;


	/**
	 * test adding mesh to scene
	 */
	cout << "--------------- Mesh Test ------------------" << endl;

	int * ids;
	int numOfMeshes = myScene.addMesh(meshName, meshType::OBJ, 0, &ids);
	cout << numOfMeshes << " meshes added to scene file with id " << ids[0] << "." << endl;

	/**
	 * test reading mesh properties from scene
	 */
	aiMesh *myMesh = myScene.getMesh(ids[0]);
	cout << "mesh " << ids[0] << " has 1st face with 2nd vertice " << myMesh->mVertices[myMesh->mFaces[0].mIndices[1]].x << ", " << myMesh->mVertices[myMesh->mFaces[0].mIndices[1]].y << ", " << myMesh->mVertices[myMesh->mFaces[0].mIndices[1]].z << endl;
	cout << "-------------------------------------------------------" << endl << endl;

	/**
	 * test adding image to scene
	 */
	cout << "--------------- Image Test ------------------" << endl;
	int id3 = myScene.addMap(imageName, mapType::JPG, 0);
	cout << "new map has id " << id3 << endl;

	/**
	 * test reading image from scene
	 */
	CImg<unsigned char>* myMap = myScene.getMap(id3);
	cout << "retrieved map " << myMap << endl;
	cout << "width is " << myMap->_width << endl;
	cout << "height is " << myMap->_height << endl;
	cout << "image " << id3 << " has first pixel color " <<  uint(myMap->atXY(0,1,0,0)) << endl;
	cout << "-------------------------------------------------------" << endl << endl;


	/**
	 * Test writing scene to file
	 */
	myScene.writeScene(sceneName);

	return 0;
}


