#include <memory>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <vector>

#include <scene-core/sra-api/sra_scene.h>
#include <scene-core/sra_implementation.h>
//#include <scene-core/XAcel.h>


namespace sra { class IScene; class IAcel; }
using namespace std;

// Unit test for saving scene file.

/********************************************************************************************/

int main(int argc, char **argv) {

	PScene sc = sra::make_scene_representation(".");   // std::shared_ptr<sra::IScene>

	sra::oid_t idGenerator = 2; // starts from 2 because id 1 is reserved for globals
	PAcel globals = sc->GetGlobals();

	// add acel for value tests
	PAcel acelVal = globals->AddChild(idGenerator++);
	acelVal->SetValue()->SetFloat(50.3);

	cout << globals->List()->Get(0)->Value()->GetFloat() << endl;

	// test byte array in an acel
	PAcel child = acelVal->AddChild(idGenerator++);
	unsigned char* buf = new unsigned char[3];
	buf[0] = 'a'; buf[1] = 'b'; buf[2] = 'c';
	child->SetValue()->SetByteBuffer(sra::byte_buffer_t(3, buf));

	sra::byte_buffer_t buf2 = globals->List()->Get(0)->List()->Get(0)->Value()->GetByteBuffer();
	cout << buf2.Ptr() << endl;

	// add multiple properties
	PAcel acelProperties = globals->AddChild(idGenerator++);

	try {
		child = acelProperties->AddProperty("prop1", idGenerator++);
		child->SetValue()->SetInt(1);
		child = acelProperties->AddProperty("prop2", idGenerator++);
		child->SetValue()->SetInt(2);
	}
	catch(sra::ExistingPropertyError& e) {
		cout << "Adding already defined property." << endl;
	}

	try {
		cout << "Id: " << globals->List()->Get(1)->Property("prop1")->Id() << " val: " << globals->List()->Get(1)->Property("prop1")->Value()->GetInt() << endl;
		cout << "Id: " << globals->List()->Get(1)->Property("prop2")->Id() << " val: " << globals->List()->Get(1)->Property("prop2")->Value()->GetInt() << endl;
	}
	catch(sra::UnsupportedPropertyError& e) {
		cout << "Accessing undefined property." << endl;
	}

	try {
		sra::set_scene_representation("savetest.scene", sc);
	}
	catch(std::invalid_argument& e) {
		cout << "Cannot save the scene file: " << e.what() << endl;
	}

	return 0;
}

