#include <memory>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <vector>

#include <scene-core/sra-api/sra_scene.h>
#include <scene-core/sra_implementation.h>
#include <scene-core/ErrorAcelValue.h>
#include <scene-core/XAcelValue.h>
#include "accessors.h"

namespace sra { class IScene; class IAcel; }
using namespace std;

void PrintCamera(VisionCameraAcelAccessor &cam) {

	cout << "width = " << cam.width() << endl;
	cout << "height = " << cam.height() << endl;
	cout << "fx = " << cam.fx() << endl;
	cout << "fy = " << cam.fy() << endl;
	cout << "cx = " << cam.cx() << endl;
	cout << "cy = " << cam.cy() << endl;
	cout << "R = ";
	for(int i = 0; i < 3; i++) {
		for(int j = 0; j < 3; j++) {
			cout << cam.R().Ptr()[3*i + j] << " ";
		}
		cout << endl;
	}

	cout << "t = ";
	for(int i = 0; i < 3; i++) cout << cam.t().Ptr()[i] << " ";
	cout << endl;
	return;
}

/********************************************************************************************/

int SaveCameras(string filename, vector<PAcel> &cameras) {

	std::ofstream out(filename.c_str());

	if(!out.is_open()) {
		cout << "Unable to save: " << filename << endl;
		return -1;
	}

	out << cameras.size() << " 0" << endl << endl;


	for(vector<PAcel>::iterator it = cameras.begin(); it != cameras.end(); ++it) {
		VisionCameraAcelAccessor cam(*it);

		out << "0 " << (cam.height() - 1);
		out << " 0 " << (cam.width() - 1) << endl;
		out << cam.fx() << " " << cam.fy() << " " << cam.cx() << " " << cam.cy() << endl;
		for(int k = 0; k < 3; k++) {
			for(int j = 0; j < 3; j++) {
				out << cam.R().Ptr()[3*k + j] << " ";
			}
			out << endl;
		}
		for(int j = 0; j < 3; j++) {
			out << cam.t().Ptr()[j] << " ";
		}
		out << endl << endl;
	}

	out.close();

	return 0;
}

/********************************************************************************************/

int main(int argc, char **argv) {

	std::shared_ptr<sra::IScene> sc = sra::get_scene_representation(argv[1]);

	PAcel cameras = sc->GetGlobals()->Property("cameras");

	vector<PAcel> camerasVector;
	for(int i = 0; i < cameras->List()->Count(); i++) {
		PAcel c = cameras->List()->Get(i);
		camerasVector.push_back(c);
		VisionCameraAcelAccessor cam(c);
		PrintCamera(cam);
	}

	if(SaveCameras(argv[2], camerasVector) != 0 ) return -1;


	return 0;
}

