
#include <cstdio>
#include <fcntl.h>
#include <fstream>
#include <stdexcept>
#include <iostream>

#include "scene-persistence/autogen/scenedb.pb.h"
#include "scene-persistence/autogen/stream_fsio.pb.h"


/* In this test we create a Scene SRA file using the protobuf API. 
 * This implies that so far, we don't use anything of SRA API. * 
 *
 * The scene file just contains links to the data. The actual data
 * are sequence of frames in the hard drive. */
int main(void)
{
	// create PB messages
	pb::Scene scene;

	// data is in the form of frame sequences, so we must specify 
	// first and last frame
	int first_frame = 363015;
	int last_frame =  363018;

	// path to the scene's data root folder
	std::string work_dir = "../../sample_data/laura/";

	// we used named streams to store different streams in the same scene file

	// rgb stream
	scene.add_streams();
	scene.mutable_streams(0)->set_video(work_dir + "%06d.exr");
	scene.mutable_streams(0)->SetExtension(fsio::name, "rgb");
	scene.mutable_streams(0)->SetExtension(fsio::first_frame, first_frame);
	scene.mutable_streams(0)->SetExtension(fsio:: last_frame,  last_frame);
	scene.mutable_streams(0)->AddExtension(fsio::exr_channel, "B");
	scene.mutable_streams(0)->AddExtension(fsio::exr_channel, "G");
	scene.mutable_streams(0)->AddExtension(fsio::exr_channel, "R");

	// alpha stream
	scene.add_streams();
	scene.mutable_streams(1)->set_video(work_dir + "%06d.exr");
	scene.mutable_streams(1)->SetExtension(fsio::name, "alpha");
	scene.mutable_streams(1)->SetExtension(fsio::first_frame, first_frame);
	scene.mutable_streams(1)->SetExtension(fsio:: last_frame,  last_frame);
	scene.mutable_streams(1)->AddExtension(fsio::exr_channel, "A");

	// ToF depth stream
	scene.add_streams();
	scene.mutable_streams(2)->set_video(work_dir + "%06d.exr");
	scene.mutable_streams(2)->SetExtension(fsio::name, "tof_depth");
	scene.mutable_streams(2)->SetExtension(fsio::first_frame, first_frame);
	scene.mutable_streams(2)->SetExtension(fsio:: last_frame,  last_frame);
	scene.mutable_streams(2)->AddExtension(fsio::exr_channel, "depth.Z");

	// ToF amplitude stream
	scene.add_streams();
	scene.mutable_streams(3)->set_video(work_dir + "%06d.exr");
	scene.mutable_streams(3)->SetExtension(fsio::name, "tof_amplitude");
	scene.mutable_streams(3)->SetExtension(fsio::first_frame, first_frame);
	scene.mutable_streams(3)->SetExtension(fsio:: last_frame,  last_frame);
	scene.mutable_streams(3)->AddExtension(fsio::exr_channel, "tof.amplitude");

	// create scene file
	std::fstream bin_output("./laura.scene", std::ios::out | std::ios::binary);
	scene.SerializeToOstream(&bin_output);
	bin_output.close();

	// create text description of scene file - it is usefull
	std::fstream txt_output("./laura.scene.txt", std::ios::out);
	txt_output << scene.DebugString() ;
	txt_output.close();


	return 0;
}
