#pragma once
#include <vector>
#include <string>
#include <scene-core/sra-api/sra_scene.h>
#include <scene-core/XAcelValue.h>

template <typename T>
inline const T& convert(const T& pos )
{
  return pos;
}
template <typename T>
inline T& convert(T& pos )
{
  return pos;
}

inline std::vector<float> convert( const sra::float3_t& vin )
{
  std::vector<float> vout(3);
  vout[0] = vin.x; vout[1] = vin.y; vout[2] = vin.z;
  return vout;
}


class GeometryAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  GeometryAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  int id() const {  return convert(acel->Property("id")->Value()->GetInt()); }
  sra::float_buffer_t vertices() const {  return convert(acel->Property("vertices")->Value()->GetFloatBuffer()); }
  sra::float_buffer_t normals() const {  return convert(acel->Property("normals")->Value()->GetFloatBuffer()); }
  sra::float_buffer_t texcoord() const {  return convert(acel->Property("texcoord")->Value()->GetFloatBuffer()); }
  sra::int_buffer_t indices() const {  return convert(acel->Property("indices")->Value()->GetIntBuffer()); }
  int texture_id() const {  return convert(acel->Property("texture_id")->Value()->GetInt()); }
};

class VectorAcelAcessor
{
  std::shared_ptr<sra::IAcel> acel;
public:
  VectorAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  float x() const {  return convert(acel->Property("x")->Value()->GetFloat()); }
  float y() const {  return convert(acel->Property("y")->Value()->GetFloat()); }
  float z() const {  return convert(acel->Property("z")->Value()->GetFloat()); }
};
class TextureDepthAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  TextureDepthAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  sra::float_buffer_t float_values() const {  return convert(acel->Property("float_values")->Value()->GetFloatBuffer()); }
  int width() const {  return convert(acel->Property("width")->Value()->GetInt()); }
  int height() const {  return convert(acel->Property("height")->Value()->GetInt()); }
};
class TextureRGBAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  TextureRGBAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  int id() const {  return convert(acel->Property("id")->Value()->GetInt()); }
  sra::byte_buffer_t values() const {  return convert(acel->Property("values")->Value()->GetByteBuffer()); }
  int width() const {  return convert(acel->Property("width")->Value()->GetInt()); }
  int height() const {  return convert(acel->Property("height")->Value()->GetInt()); }
};
class CameraAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  CameraAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  float xfov() const {  return convert(acel->Property("xfov")->Value()->GetFloat()); }
  float aspect_ratio() const {  return convert(acel->Property("aspect_ratio")->Value()->GetFloat()); }
  float znear() const {  return convert(acel->Property("znear")->Value()->GetFloat()); }
  float zfar() const {  return convert(acel->Property("zfar")->Value()->GetFloat()); }
  std::vector<float> position() const {  return convert(acel->Property("position")->Value()->GetFloat3()); }
  std::vector<float> direction() const {  return convert(acel->Property("direction")->Value()->GetFloat3()); }
  std::vector<float> up() const {  return convert(acel->Property("up")->Value()->GetFloat3()); }
};
class SpotLightAcelAcessor 
{
  std::shared_ptr<sra::IAcel> acel;
public:
  SpotLightAcelAcessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  float falloff_angle() const {  return convert(acel->Property("falloff_angle")->Value()->GetFloat()); }
  std::vector<float> position() const {  return convert(acel->Property("position")->Value()->GetFloat3()); }
  std::vector<float> direction() const {  return convert(acel->Property("direction")->Value()->GetFloat3()); }
  std::vector<float> up() const {  return convert(acel->Property("up")->Value()->GetFloat3()); }
};

class VisionCameraAcelAccessor
{
  std::shared_ptr<sra::IAcel> acel;
public:
  VisionCameraAcelAccessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  float fx() const {  return convert(acel->Property("fx")->Value()->GetFloat()); }
  float fy() const {  return convert(acel->Property("fy")->Value()->GetFloat()); }
  float cx() const {  return convert(acel->Property("cx")->Value()->GetFloat()); }
  float cy() const {  return convert(acel->Property("cy")->Value()->GetFloat()); }

  int width() const {  return convert(acel->Property("width")->Value()->GetInt()); }
  int height() const {  return convert(acel->Property("height")->Value()->GetInt()); }
  sra::float_buffer_t R() const {  return convert(acel->Property("R")->Value()->GetFloatBuffer()); }
  sra::float_buffer_t t() const {  return convert(acel->Property("t")->Value()->GetFloatBuffer()); }
};

class TCMeshAcelAccessor
{
  std::shared_ptr<sra::IAcel> acel;
public:
  TCMeshAcelAccessor(std::shared_ptr<sra::IAcel> a) : acel(a) {}

  int frame() const {  return convert(acel->Property("frame")->Value()->GetInt()); }
  sra::int_buffer_t vertex_ids() const {  return convert(acel->Property("vertex_ids")->Value()->GetIntBuffer()); }
  sra::float_buffer_t vertex_positions() const {  return convert(acel->Property("vertex_positions")->Value()->GetFloatBuffer()); }
  sra::int_buffer_t faces() const {  return convert(acel->Property("faces")->Value()->GetIntBuffer()); }
};
